import { RandomResults } from './mevaTrackLinking.service';
import girderRest from '../plugins/girder';

export interface LinkingFeature {
    linked: boolean;
    sourceGirderId: string;
    sourceTrackId: string;
    sourceClipFilename: string;
    linkedActor: string; //`${actorID}-${date}-${outfitId}`
    textFeatures: string;
    properties: string[];
    linkReason: string;
    featureDrawings: {identifier: string; data: string}[]; // base64 encodings of the images
    time?: number;
}


interface LinkingListActorData {
    actorId: string;
    date: string;
    outfitId: string;
}

export interface LinkingListItem {
    sourceClipFilename: string;
    sourceTrackId: string;
    Actors: LinkingListActorData[];
}
const baseEndpoint = 'linked_feature';
async function getFeatureLinking(
  folder: string,
): Promise<RandomResults | null> {
  return (await girderRest.get<RandomResults | null>(`${baseEndpoint}/${folder}/linking_item`, {
  })).data;
}

async function getFeatureProperties(folder: string): Promise<string[]> {
  return (await girderRest.get<string[]>(`${baseEndpoint}/${folder}/linking_properties`, {
  })).data;
}

async function addFeatureProperty(folder: string, property: string): Promise<string[]> {
  return (await girderRest.post<string[]>(`${baseEndpoint}/${folder}/linking_properties`, null, {
    params: {
      property,
    },
  })).data;
}

// async function addFeature(folder: string, data: LinkingFeature,): Promise<boolean> {
//     //We need to upload image data first
// }


// Admin Actions

async function initLinkingFolder(folder: string): Promise<Record<string, string>> {
  return (await girderRest.post<Record<string, string>>(`${baseEndpoint}/${folder}/init`, {})).data;
}

async function deleteLinkingFeatures(folder: string): Promise<boolean> {
  return (await girderRest.delete<boolean>(`${baseEndpoint}/${folder}`)).data;
}

async function setLinkingList(folder: string, data: LinkingListItem[]): Promise<boolean> {
  return (await girderRest.post<boolean>(`${baseEndpoint}/${folder}/linkinglist`, {
    params: data,
  })).data;
}

async function getLinkingList(folder: string): Promise<LinkingListItem[]> {
  return (await girderRest.get<LinkingListItem[]>(`${baseEndpoint}/${folder}/linkinglist`)).data;
}

async function addLinkingFeature(folder: string, data: LinkingFeature): Promise<LinkingFeature> {
  return (await girderRest.put(`${baseEndpoint}/${folder}`, data)).data;
}


async function exportLinkingFeatures(folderId: string) {
  return (girderRest.post(
    `${baseEndpoint}/${folderId}/export_linked_features`, {},
  ));
}
async function getExportedFeatures(folderId: string): Promise<string> {
  return (await girderRest.get(
    `${baseEndpoint}/${folderId}/exported_linked_features`, {},
  )).data;
}

async function createLinkingList(folderId: string, limit: number, totalActors: number):
Promise<string> {
  return (girderRest.post(
    `${baseEndpoint}/${folderId}/generate_linking_list?limit=${limit}&totalActors=${totalActors}`, {},
  ));
}

async function getUserProgress(folderId: string):
Promise< {total: number; users: Record<string, number> }> {
  return (await girderRest.get(
    `${baseEndpoint}/${folderId}/user_progress`, {},
  )).data;
}

export {
  addLinkingFeature,
  getFeatureLinking,
  getFeatureProperties,
  addFeatureProperty,
  // Admin Endpoints
  initLinkingFolder,
  deleteLinkingFeatures,
  setLinkingList,
  getLinkingList,
  exportLinkingFeatures,
  getExportedFeatures,
  createLinkingList,
  getUserProgress,
};
