import { MevaFilter } from 'dive-common/apispec';
import girderRest from '../plugins/girder';

interface HTMLFile extends File {
    webkitRelativePath?: string;
}

const baseEndpoint = 'meva_checkin';

export interface Actors {
    pageResults: Record<string, Actor>;
    sourceTrack: Record<string, Actor>;
    totalPages: number;
}

export interface Actor {
    imageIds: string[];
    actorId: string;
    date: string;
}

async function createActorFolder(baseId: string) {
  const createActor = await girderRest.post(`${baseEndpoint}/${baseId}/create_actor_folder`);
  if (createActor.status === 200) {
    return createActor.data;
  }
  return '';
}
async function createGPSFolder(baseId: string) {
  const createGPS = await girderRest.post(`${baseEndpoint}/${baseId}/create_gps_folder`);
  if (createGPS.status === 200) {
    return createGPS.data;
  }
  return '';
}

async function importActors(folderId: string) {
  const final = await girderRest.post(`${baseEndpoint}/process_actor_zip/${folderId}`, null, {
  });
  if (final.status === 200) {
    return true;
  }
  return false;
}

async function importGPS(folderId: string) {
  const final = await girderRest.post(`${baseEndpoint}/process_gps_zip/${folderId}`, null, {
  });
  if (final.status === 200) {
    return true;
  }
  return false;
}

async function getActors(
  offset = 0,
  pageSize = 10,
  filters: MevaFilter,
): Promise<Actors> {
  return (await girderRest.get<Actors>(`${baseEndpoint}/get_actors?offset=${offset}&pageSize=${pageSize}`, {
    params: {
      filters,
    },
  })).data;
}

export {
  createActorFolder,
  createGPSFolder,
  importActors,
  importGPS,
  getActors,
};
