import Vue from 'vue';
import VueCompositionApi from '@vue/composition-api';

import registerNotifications from 'vue-media-annotator/notificatonBus';
import snackbarService from 'dive-common/vue-utilities/snackbar-service';
import promptService from 'dive-common/vue-utilities/prompt-service';
import vMousetrap from 'dive-common/vue-utilities/v-mousetrap';

import getVuetify from './plugins/vuetify';
import girderRest from './plugins/girder';
import App from './App.vue';
import router from './router';
import store from './store';

Vue.config.productionTip = false;
Vue.use(VueCompositionApi);
Vue.use(vMousetrap);


Promise.all([
  store.dispatch('Brand/loadBrand'),
  girderRest.fetchUser(),
]).then(() => {
  const vuetify = getVuetify(store.state.Brand.brandData?.vuetify);
  Vue.use(snackbarService(vuetify));
  Vue.use(promptService(vuetify));
  new Vue({
    router,
    store,
    vuetify,
    provide: {
      girderRest,
      notificationBus: girderRest, // gwc.JobList expects this
      vuetify,
    },
    render: (h) => h(App),
  })
    .$mount('#app')
    .$snackbarAttach()
    .$promptAttach();

  /** Start notification stream if everything else succeeds */
  registerNotifications(girderRest).connect();
});
