import Vue from 'vue';
import Router from 'vue-router';

import girderRest from './plugins/girder';
import Home from './views/Home.vue';
import Meva from './views/Meva.vue';
import MevaFolders from './views/MevaFolders.vue';
import AllLinkedTracks from './views/AllLinkedTracks.vue';
import Jobs from './views/Jobs.vue';
import Login from './views/Login.vue';
import RouterPage from './views/RouterPage.vue';
import MevaLoader from './views/MevaLoader.vue';
// import Settings from './views/Settings.vue'; // Remove for now
import ViewerLoader from './views/ViewerLoader.vue';
import MultiViewLoader from './views/MultiViewLoader.vue';
import ChipList from './views/ChipList.vue';
import Actors from './views/Actors.vue';
import Random from './views/Random.vue';
import BriarLinking from './views/BriarLinking.vue';
import ActorLinks from './views/ActorLinks.vue';
import AdminPage from './views/AdminPage.vue';

Vue.use(Router);

function beforeEnter(to, from, next) {
  if (!girderRest.user) {
    next('/login');
  } else {
    next();
  }
}

const router = new Router({
  routes: [
    {
      path: '/login',
      name: 'login',
      component: Login,
    },
    {
      path: '/viewer/:id',
      name: 'viewer',
      component: ViewerLoader,
      props: true,
      beforeEnter,
    },
    {
      path: '/meva',
      props: true,
      component: MevaLoader,
      beforeEnter,
      children: [
        {
          path: '/',
          name: 'mevaFolders',
          props: true,
          component: MevaFolders,
          beforeEnter,
        },
        {
          path: ':id',
          name: 'mevaFolder',
          props: true,
          component: Meva,
          beforeEnter,
        },
        {
          path: ':id/actors',
          name: 'actors',
          props: true,
          component: Actors,
          beforeEnter,
        },
        {
          path: ':id/admin',
          name: 'actors',
          props: true,
          component: AdminPage,
          beforeEnter,
        },
        {
          path: ':id/chiplist/:folderId',
          name: 'chiplist',
          props: true,
          component: ChipList,
          beforeEnter,
        },
        {
          path: ':id/allLinked',
          name: 'allLinked',
          component: AllLinkedTracks,
          beforeEnter,
        },
        {
          path: ':id/briarlinking',
          name: 'briarlinking',
          component: BriarLinking,
          props: true,
          beforeEnter,
        },
        {
          path: ':id/random',
          name: 'random',
          component: Random,
          props: true,
          beforeEnter,
        },
        {
          path: ':id/actorLinks/:actorId/:date/:outfitId',
          name: 'actorLinks',
          component: ActorLinks,
          props: true,
          beforeEnter,
        },

      ],
    },

    {
      path: '/',
      name: 'router_base',
      props: (route) => ({ hideNav: route.query.hideNav === 'true' }),
      component: RouterPage,
      children: [
        /*
         * Deprecated and removed for now
        {
          path: 'settings',
          name: 'settings',
          component: Settings,
          beforeEnter,
        },
        */
        {
          path: '/multiview/:params',
          name: 'multiview',
          component: MultiViewLoader,
          props: true,
          beforeEnter,
        },
        {
          path: 'jobs',
          name: 'jobs',
          component: Jobs,
          beforeEnter,
        },
        {
          path: ':_modelType?/:_id?',
          name: 'home',
          component: Home,
          beforeEnter,
        },

      ],
    },
  ],
});

export default router;
