import { merge } from 'lodash';
import { Module } from 'vuex';

import { BrandData, getBrandData } from 'platform/web-girder/api/meva.service';
import defaultLogo from 'dive-common/assets/logo.png';

import { BrandState, RootState } from './types';

function setFavicon(href: string) {
  let faviconLink = document.querySelector("link[rel~='icon']");
  if (!faviconLink) {
    faviconLink = document.createElement('link');
    faviconLink.setAttribute('rel', 'icon');
    document.getElementsByTagName('head')[0].appendChild(faviconLink);
  }
  faviconLink.setAttribute('href', href);
}

function setTitle(title: string) {
  const titleEl = document.querySelector('title');
  if (titleEl) {
    titleEl.innerText = title;
  }
}

const brandModule: Module<BrandState, RootState> = {
  namespaced: true,
  state: {
    brandData: {
      vuetify: null,
      favicon: null,
      logo: defaultLogo,
      name: 'MEVA Person Search Tool',
      loginMessage: 'MEVA is a person search tool',
    },
  },
  mutations: {
    setBrandData(state, data: BrandData) {
      state.brandData = merge(state.brandData, data);
      setTitle(state.brandData.name);
      if (state.brandData.favicon !== null) {
        setFavicon(state.brandData.favicon);
      }
    },
  },
  actions: {
    async loadBrand({ commit }) {
      commit('setBrandData', await getBrandData());
    },
  },
};

export default brandModule;
