# Docker Documentation

MEVA Person Search is easiest to set up with docker.

## docker-compose

We recommend running MEVA Person Search with docker-compose. Clone this repository and configure options in `.env`.

``` bash
# Change to correct directory
cd docker/

# Initiate the .env file
cp .env.default .env

# Pull pre-built images
docker-compose pull

# Bring the services up
docker-compose up
```

MEVA Person Search server will be running at http://localhost:8010/

You can run the data viewer without needing GPU support as well by using the `docker-compose.cpu.yml` file
There exists an Nvidia based docker-compose file which may be used in the future to provide integrated tracking on files.  Currently it is reccommended to only use the CPU only version.  The GPU (Nvidia based) docker-compose file can be used if your system has a supported GPU, it won't provide any additional functionality currently.

``` bash
docker-compose up girder
```

## Images

A DIVE deployment consists of 2 main services.

* meva-web- the web server
* meva-worker - the queue worker

In addition, a database (MongoDB) and a queue service (RabbitMQ) are required.

### Girder Web Server

This image contains both the backend and client.

| Variable | Default | Description |
|----------|---------|-------------|
| GIRDER_MONGO_URI | mongodb://mongo:27017/girder | a mongodb connection string |
| GIRDER_ADMIN_USER | admin | admin username |
| GIRDER_ADMIN_PASS | letmein | admin password |
| CELERY_BROKER_URL | amqp://guest:guest@rabbit/ | rabbitmq connection string |
| BROKER_CONNECTION_TIMEOUT | 2 | rabbitmq connection timeout |

[Read more about configuring girder.](https://girder.readthedocs.io/en/latest/)

### Girder Worker

This image contains a celery worker to run transcoding jobs and possible future person tracking.

| Variable | Default | Description |
|----------|---------|-------------|
| CELERY_BROKER_URL | amqp://guest:guest@rabbit/ | rabbitmq connection string |
| BROKER_CONNECTION_TIMEOUT | 2 | rabbitmq connection timeout |

## Build your own images

> **Note:** In order to build images yourself, the `.git` folder must exist, so you must `git clone` from source control. 

``` bash
docker-compose build
```

## Production deployment

The system is designed to be deployed either with a self signed certificate for https or using the ACME server and traefik's built in Let's Encrypt.

### Automatic SSL
``` bash
docker-compose -f docker-compose.cpu.yml -f docker-compose.prod.yml up -d
```

### Self Signed SSL Certificate

This deployment will use a self signed certificate instead of relying on traefik and letsecrypt to generate one.  This would be used if the system is hosted on an internal VPN without public access.

This deployment relies on a `certs` folder located at `./docker/certs` within that folder should be two files associated with the self signed certificate:
`./certs/cert.crt`
`./certs/cert.key`

``` bash
docker-compose -f docker-compose.cpu.yml -f docker-compose.local.https.yml up -d
```

