FROM node:16 as builder
WORKDIR /app

# Install dependencies
COPY client/package.json client/yarn.lock /app/
RUN yarn --frozen-lockfile
# Build
COPY client/ /app/
RUN yarn build:web


FROM girder/girder:3.1.5 as runtime

# install tini init system
ENV TINI_VERSION v0.19.0
ADD https://github.com/krallin/tini/releases/download/${TINI_VERSION}/tini /tini
RUN chmod +x /tini

ENV GIRDER_MONGO_URI mongodb://mongo:27017/girder
ENV GIRDER_ADMIN_USER admin
ENV GIRDER_ADMIN_PASSWORD viame
ENV CELERY_BROKER_URL amqp://guest:guest@rabbit/
ENV BROKER_CONNECTION_TIMEOUT 2
ENV MEVA_DATABASE_URI sqlite:////data/db/meva.db

# Initialize python virtual environment
RUN apt-get update && apt-get install -y python3.7-venv
RUN apt install -y libgl1-mesa-dev
ENV VIRTUAL_ENV=/opt/venv
RUN python3.7 -m venv $VIRTUAL_ENV

# this will activate the virtual env
ENV PATH="$VIRTUAL_ENV/bin:$PATH"

# Cryptography requires latest pip, setuptools.
# https://cryptography.io/en/latest/faq.html#installing-cryptography-fails-with-error-can-not-find-rust-compiler
RUN pip install -U pip
RUN pip install setuptools==66.0.0

COPY --from=builder /app/dist/ $VIRTUAL_ENV/share/girder/static/meva/

WORKDIR /home/meva_girder

COPY docker/provision /home/provision
COPY server/setup.py /home/meva_girder/
RUN chmod -R 777 /home/meva_girder/

RUN pip install --no-cache-dir .
COPY server/ /home/meva_girder/
RUN pip install --no-deps .

COPY package.json.template /opt/venv/lib/python3.7/site-packages/girder/web_client
RUN girder build

ENTRYPOINT [ "/home/provision/girder_entrypoint.sh" ]
