FROM nvidia/cuda:10.1-cudnn8-devel-ubuntu18.04
#FROM ubuntu:18.04

USER root

WORKDIR /home

RUN apt-get update \
  && apt-get install -y git

ENV PATH=/usr/local/cuda/bin:$PATH
ENV LD_LIBRARY_PATH=/usr/local/cuda/lib64:$LD_LIBRARY_PATH

ENV NVIDIA_DRIVER_CAPABILITIES=compute,utility
ENV NVIDIA_VISIBLE_DEVICES=all
# install tini init system
ENV TINI_VERSION v0.19.0
ADD https://github.com/krallin/tini/releases/download/${TINI_VERSION}/tini /tini
RUN chmod +x /tini

# VIAME install at /opt/noaa/viame/
# VIAME pipelines at /opt/noaa/viame/configs/pipelines/

ENV CELERY_BROKER_URL amqp://guest:guest@rabbit/
ENV BROKER_CONNECTION_TIMEOUT 2

WORKDIR /home/meva_girder

# BEGIN: Porting girder worker install from girder/girder_worker Dockerfile.py3
RUN apt-get update && \
	export DEBIAN_FRONTEND=noninteractive && \
  apt-get install -qy software-properties-common python3-software-properties && \
  apt-get update && apt-get install -qy \
    build-essential \
    wget \
    python3.7 \
    locales \
    python3.7-tk \
    r-base \
    libffi-dev \
    libyaml-dev \
    libssl-dev \
    libjpeg-dev \
    zlib1g-dev \
    r-base \
    ffmpeg \
    curl \
    libpython3.7-dev && \
  apt-get clean && rm -rf /var/lib/apt/lists/*

RUN wget https://bootstrap.pypa.io/get-pip.py && python3.7 get-pip.py
# END port of worker installation

# Initialize python virtual environment
RUN apt-get update && apt-get install -y python3.7-venv



# Switch over to user "worker" 1099:1099 to align with base image
# https://github.com/VIAME/VIAME/blob/master/cmake/build_server_docker.sh#L123
RUN useradd --create-home --uid 1099 --shell=/bin/bash worker
RUN mkdir /home/AlphaPose
RUN chown -R worker:worker /home/AlphaPose
USER worker
WORKDIR /home
RUN git clone https://github.com/MVIG-SJTU/AlphaPose.git
WORKDIR /home/AlphaPose
RUN git config --global user.email "Bryon.lewis@kitware.com"
RUN git config --global user.name "Bryon Lewis"
RUN git pull --no-edit origin pull/592/head
WORKDIR /home/meva_girder
COPY --chown=worker:worker /AlphaPose /home/AlphaPose

RUN ls /home/AlphaPose/pretrained_models

ENV VIRTUAL_ENV=/home/worker/venv
RUN python3.7 -m venv $VIRTUAL_ENV

# Activate the venv
ENV PATH="$VIRTUAL_ENV/bin:$PATH"


# install nodePos
ENV PATH=/usr/local/cuda/bin:$PATH
ENV LD_LIBRARY_PATH=/usr/local/cuda/lib64:$LD_LIBRARY_PATH

WORKDIR /home/AlphaPose
RUN pip install torch==1.5.1 \
    torchvision==0.6.1 \
    Cython \
    numpy==1.14.5 \
    scipy==1.1.0 \
    opencv_python==3.4.2.17 \
    opencv_contrib_python==3.4.2.17 \
    matplotlib==2.2.2 \
    tqdm==4.23.4 \
    Image==1.5.25 \
    Pillow==5.3.0 \
    munkres==1.0.12 \
    natsort \
    tk-tools \
    easydict \
    pycocotools \
    ninja 
RUN ls
RUN python3 -m pip install --user --upgrade pip
RUN curl https://bootstrap.pypa.io/get-pip.py -o get-pip.py
RUN python3 get-pip.py --force-reinstall
ENV LANG=C.UTF-8
RUN ls /usr/local/cuda/lib64/
ENV TORCH_CUDA_ARCH_LIST="3.5;3.7;5.0;5.2;6.0+PTX;6.1+PTX;7.0+PTX;7.5+PTX"
RUN python3 setup.py build develop

WORKDIR /home/meva_girder

# Create directory for addons
RUN mkdir -p /tmp/addons


# Cryptography requires latest pip, setuptools.
# https://cryptography.io/en/latest/faq.html#installing-cryptography-fails-with-error-can-not-find-rust-compiler
RUN pip install -U pip
RUN pip install setuptools==66.0.0

# Pip install dependencies
COPY --chown=worker:worker server/setup.py /home/meva_girder/
RUN pip install --no-cache-dir .

# Pip install actual packages
COPY --chown=worker:worker server/ /home/meva_girder/
RUN pip install --no-deps .

# Copy provision scripts
COPY --chown=worker:worker docker/provision /home/provision

ENTRYPOINT ["/tini", "--"]
CMD ["/home/provision/girder_worker_entrypoint.sh"]
