FROM ubuntu:18.04

USER root

WORKDIR /home

RUN apt-get update \
  && apt-get install -y git

# install tini init system
ENV TINI_VERSION v0.19.0
ADD https://github.com/krallin/tini/releases/download/${TINI_VERSION}/tini /tini
RUN chmod +x /tini

# VIAME install at /opt/noaa/viame/
# VIAME pipelines at /opt/noaa/viame/configs/pipelines/

ENV CELERY_BROKER_URL amqp://guest:guest@rabbit/
ENV BROKER_CONNECTION_TIMEOUT 2
ENV MEVA_DATABASE_URI sqlite:////data/db/meva.db


# BEGIN: Porting girder worker install from girder/girder_worker Dockerfile.py3
RUN apt-get update && \
	export DEBIAN_FRONTEND=noninteractive && \
  apt-get install -qy software-properties-common python3-software-properties && \
  apt-get update && apt-get install -qy \
    build-essential \
    wget \
    python3.7 \
    locales \
    python3.7-tk \
    r-base \
    libffi-dev \
    libyaml-dev \
    libssl-dev \
    libjpeg-dev \
    zlib1g-dev \
    r-base \
    ffmpeg \
    curl \
    libpython3.7-dev && \
  apt-get clean && rm -rf /var/lib/apt/lists/*

RUN wget https://bootstrap.pypa.io/get-pip.py && python3.7 get-pip.py
# END port of worker installation

# Initialize python virtual environment
RUN apt-get update && apt-get install -y python3.7-venv

RUN useradd --create-home --uid 1099 --shell=/bin/bash worker

ENV VIRTUAL_ENV=/home/worker/venv
RUN python3.7 -m venv $VIRTUAL_ENV

# Activate the venv
ENV PATH="$VIRTUAL_ENV/bin:$PATH"


WORKDIR /home/meva_girder
# Create directory for addons
RUN mkdir -p /tmp/addons


# Cryptography requires latest pip, setuptools.
# https://cryptography.io/en/latest/faq.html#installing-cryptography-fails-with-error-can-not-find-rust-compiler
RUN pip install -U pip
RUN pip install setuptools==66.0.0 wheel
RUN pip install numpy==1.14.5 \
    opencv_python==3.4.2.17 \
    Pillow==5.3.0

# Pip install dependencies
COPY --chown=worker:worker server/setup.py /home/meva_girder/
RUN chown -R worker:worker /home/worker/venv/lib/python3.7/site-packages/
RUN chmod -R 777 /home/meva_girder/
RUN ls -l /home/
RUN pip install --no-cache-dir .

# Pip install actual packages
COPY --chown=worker:worker server/ /home/meva_girder/
COPY --chown=worker:worker scripts/ /home/data_scripts/

RUN pip install --no-deps .

USER worker

# Copy provision scripts
COPY --chown=worker:worker docker/provision /home/provision

ENTRYPOINT ["/tini", "--"]
CMD ["/home/provision/girder_worker_entrypoint.sh"]
