# MEVA FairMOT Open Tracker

The code contained in this directory includes the FairMOT based tracker `mevaTracker.py`. This script has a number of requirements that need to be specially built and coordinated to work properly.  This is managed through a docker image that can be built with the code in this repository. Please follow the docker build instructions to build this code locally.

## Background

There are three images used in producing the final docker image used for tracking, they are:

1. `gitlab.kitware.com:4567/afrl_act3/meva-person-search/artifacts`
2. `gitlab.kitware.com:4567/afrl_act3/meva-person-search/vendor`
3. `gitlab.kitware.com:4567/afrl_act3/meva-person-search/fairmot_tracker`

Each image manages a different part of the final image which is `fairmot_tracker`. First, `artifacts`contains the required `pt` files for use with the tracker. In general you should not need to build this container,  but instead can pull it from the docker registry associated with the proejct.  Second, the`vendor`image builds and installs the code required for the script to run. Finally the `fiarmot_tracker` image copies in the `mevaTracker.py` script and sets the `PYTHONPATH` variable to handle some import path details required by the FairMOT code.

## Building the Artifact Image
In general it should not be nessisary to build the `artifact` image. In general you should pull the image locally from the docker registry associated with the project.

``` sh
# Credentials should be your gitlab.kitware.com credentials
docker login gitlab.kitware.com:4567
docker pull gitlab.kitware.com:4567/afrl_act3/meva-person-search/artifacts:latest
```

If you must build `artifacts` you can use the following instructions:

``` sh
cd artifacts/
./build.sh
```

The `build.sh` script will check to make sure you have the required files (which are currently git ignored) in the `artifacts/` directory. Please contact Dan Davila to find where these files may be downloaded.

You may push the latest version of this image by running:

``` sh
cd artifacts/
./push.sh
```

## Building the Vendor Image

As implemented,  the vendor image must be built on a GPU machine and the GPU must be accessible _at build time_. This means you must configure your docker daemon appropriately. The following configuration _must_ be in /etc/docker/daemon.json:

``` json
{
    "runtimes": {
        "nvidia": {
            "path": "/usr/bin/nvidia-container-runtime",
            "runtimeArgs": []
         }
    },
    "default-runtime": "nvidia"
}

```

If you have to update your /etc/docker/daemon.json configuration you must restart the docker daemon:

``` sh
sudo systemctl restart docker
```

Once complete,  you can build the `vendor` image by running the following:

``` sh
cd vendor/
./build.sh
```

Note that this will tag the gitlab.kitware.com:4567/afrl_act3/meva-person-search/vendor with the `latest` tag by default.  You can change this by running the following:

``` sh
cd vendor/
IMAGE_TAG=my-tag ./build.sh
```

This is the prefered method as building locally depends on the GPU of the host machine and may not be portable across host machines.

You can push the `vendor` tag by running:

``` sh
cd vendor/
./push.sh
```

or alternately:

``` sh
cd vendor/
IMAGE_TAG=my-tag ./push.sh
```

## Building the fairmot_tracker Image

Finally,  you may build the `fairmot_tracker` image by running:

``` sh
./build.sh
```

or alternately

``` sh
IMAGE_TAG=my-tag ./build.sh
```

And you can push this image by running:

``` sh
./push.sh
```

or alternately

``` sh
IMAGE_TAG=my-tag ./push.sh
```

# Running the fairmot_tracker image

The entrypoint for the `fairmot_tracker` image is `conda run -n FairMOT python ./mevaTracker.py` which means the tracker may be run with the following command:

``` sh
docker run -it --rm --gpus=all -e PYTHONUNBUFFERED=1 \
-v /mnt/diva-nas03/meva-released/:/data \
-v /data2/projects/meva/results/:/results/ \
-v /home/local/KHQ/daniel.davila/projects/meva-data-repo/annotation/DIVA-phase-2/MEVA/:/diva_priors \
gitlab.kitware.com:4567/afrl_act3/meva-person-search/fairmot_tracker:latest tracker \
/data/2018-03-11/12/2018-03-11.11-55-00.12-00-00.school.G328.r13.avi \
--priors_path /diva_priors/ \
--frame_slice "::3:3" \
--save_images True \
--save_dir /results/ \
--tracker_det_thresh 0.4 \
--ignore_len 75 \
--ignore_iou 0.3
```

Where the tag `my-tag` should be replaced with your tagged version and `/path/to/host/data` should be replaced with the path to your hosts data. Please also make sure to reference a video at that path (i.e. replace `my_video.avi` with an actual video file).

