import json
import matplotlib.pyplot as plt
import numpy as np
from scipy.signal import savgol_filter

ann_stats_file = '/home/local/KHQ/daniel.davila/projects/meva-person-search/mevaTracker/notebooks/annotation_meta.json'

with open(ann_stats_file, 'r') as af:
    data = json.load(af)

assert(data['totalPages'] == 1)

dates = {}
dates_reviewing = {}
all_track_counts = {'all' : []}
cleaned_track_counts = {}
reviewed_track_counts = {}
names = {}
meva_dates = {}

for video in data['pageResults']:
    
    #Check for case where video has 0 cleaning time - bug with process
    if video['MEVA_Data']['CleaningTime'] == 0:
        continue
    
    reviewing = video['MEVA_State']['Reviewing']
    linking = video['MEVA_State']['Linking']
    
    beginTracks = video['MEVA_Data']['beginTracks']
    all_track_counts['all'].append(int(beginTracks))

    if reviewing:
        date = video['MEVA_Data']['CleaningTime'][:10]
        endTracks = video['MEVA_Data']['endTracks']
        clean_time = video['MEVA_Data']['CleaningDuration']
        cleaner = video['MEVA_Assignment']['Cleaner']['name']
        meva_date = video['metadata']['date']
        review_time = video['MEVA_Data']['ReviewingDuration']

        if meva_date in meva_dates:
            meva_dates[meva_date] += 1
        else:
            meva_dates[meva_date] = 1

        if date in dates:
            dates[date].append(int(clean_time))
            cleaned_track_counts[date].append(int(endTracks))
            names[date].append((cleaner, int(clean_time)))
            dates_reviewing[date].append(int(review_time))
        else:
            dates[date] = [int(clean_time)]
            cleaned_track_counts[date] = [int(endTracks)]
            names[date] = [(cleaner, int(clean_time))]
            dates_reviewing[date] = [int(review_time)]

    if linking:
        date = video['MEVA_Data']['ReviewingTime'][:10]
        endTracks = video['MEVA_Data']['endTracks']
        
        if date in reviewed_track_counts:
            reviewed_track_counts[date].append(int(endTracks))
        else:
            reviewed_track_counts[date] = [int(endTracks)]
        
        date = video['MEVA_Data']['CleaningTime'][:10]
        review_time = video['MEVA_Data']['ReviewingDuration']

full_names = names.copy()      
for date, name_list in names.items():
    names[date] = np.unique([item[0] for item in name_list])


scatter_dates = []
scatter_times = []
scatter_endTracks = []
scatter_tpt_dates = []
scatter_tpt = []

for date, times in dates.items():
    scatter_dates.extend([date]*len(times))
    scatter_times.extend(times)
    
for date, tracks in cleaned_track_counts.items():
    #scatter_dates.extend([date]*len(times))
    scatter_endTracks.extend(tracks)
    
for date, times in dates.items():
    #scatter_dates.extend([date]*len(times))
    tpt = [time/num_tracks for time, num_tracks in zip(times, cleaned_track_counts[date]) if num_tracks]
    scatter_tpt.extend(tpt)
    scatter_tpt_dates.extend([date]*len(tpt))

person_times = {}
names = ['chad.coumbes', 'clafleche', 'daniel.davila', 'kelsey.smith', 'jaefish', 'maryann.olstad', 'lauren.macpherson']

for date in full_names:
    person_times[date] = {}
    for name in names:
        person_times[date][name] = sum([item[1] for item in full_names[date] if item[0] == name])

X = sorted(list(full_names.keys()))

X_axis = np.arange(len(X))

#dates = sorted(person_times.keys())

plt.figure(figsize=(15,10))
for ii,name in enumerate(names):
    plt.bar(X_axis - ii/len(names), [person_times[date][name] for date in X], 1/len(names), label = name)
#plt.bar(X_axis + 0.2, [item[1] for item in np.concatenate(list(full_names.values())) if item[0]=='clafleche'], 0.4, label = 'Car')
#plt.xticks(rotation=45, ha='right')
plt.xticks(X_axis, X, rotation=45, ha='right')
plt.ylim(0,300)
plt.xlabel("Date")
plt.ylabel("Minutes Cleaning")
plt.title("Minutes Cleaning per Person Over Time")
plt.legend()
plt.show()

print('# all tracks:', np.hstack(all_track_counts.values()).sum())
print('# all videos:', len(np.hstack(all_track_counts.values())))

print('# cleaned tracks:', np.hstack(cleaned_track_counts.values()).sum())
print('# cleaned videos:', len(np.hstack(cleaned_track_counts.values())))

print('# reviewed tracks:', np.hstack(reviewed_track_counts.values()).sum())
print('# reviewed videos:', len(np.hstack(reviewed_track_counts.values())))
#print('Total number of video reviewed:', len(np.hstack(dates_reviewing.values())))

fontsize=60
linewidth = 5
plt.rc('font', **{'size': 20})
plt.rc('axes', linewidth=6)
fig = plt.figure(num=None, figsize=(15.3, 10.7), dpi=60)

#plt.scatter(scatter_dates, scatter_times)
plt.plot(list(dates.keys()), savgol_filter([np.mean(times) for times in dates.values()], 5, 3), linewidth=linewidth)
plt.xlabel("Date", fontsize=fontsize)
plt.ylabel('Avg Time per Video', fontsize=fontsize)
plt.xticks(rotation=45, ha='right')

fig = plt.figure(num=None, figsize=(15.3, 10.7), dpi=60)
plt.scatter(scatter_dates, scatter_endTracks)
#plt.plot(list(dates.keys()), savgol_filter([np.mean(times) for times in dates.values()], 5, 3))
plt.plot(list(dates.keys()), [np.mean(times) for times in cleaned_track_counts.values()], linewidth=linewidth)
plt.xlabel("Date", fontsize=fontsize)
plt.xticks(rotation=45, ha='right')
plt.ylabel('Tracks per Video', fontsize=fontsize)

fig = plt.figure(num=None, figsize=(15.3, 10.7), dpi=60)
plt.scatter(scatter_tpt_dates, scatter_tpt)
#plt.plot(list(dates.keys()), savgol_filter([np.mean(times) for times in dates.values()], 5, 3))
#plt.plot(list(dates.keys()), savgol_filter([np.sum(times) for times in track_counts.values()], 3, 1), linewidth=linewidth)
plt.plot(list(dates.keys()), [np.sum(times)/np.sum(tracks) for tracks, times in zip(cleaned_track_counts.values(),dates.values())], linewidth=linewidth)
plt.xlabel("Date", fontsize=fontsize)
plt.xticks(rotation=45, ha='right')
plt.ylabel('Minutes on Task per Track', fontsize=fontsize)



