import girder_client
import json
import os
import click

apiURL = "ps.mevadata.org"
rootMevaFolderId = "6138d359648e333daf451f7d"



def login():
    gc = girder_client.GirderClient(apiURL, apiRoot='girder/api/v1')
    gc.authenticate(interactive=True)
    return gc

def get_mevastate_data(gc: girder_client.GirderClient):
    data = gc.sendRestRequest('GET', f'meva_state/{rootMevaFolderId}/all_meva?limit=5000&sort=created&sortdir=1')
    pageResults = data.get('pageResults', False)
    meva_state = {}
    if pageResults is not False:
        for item in pageResults:
            meva_state[item['filename']] = item
    return meva_state

def check_filename_on_server(name, meva_state):
    if name in meva_state.keys():
        return meva_state[name]
    return None

def get_actor_links(gc: girder_client.GirderClient, limit):
    return gc.sendRestRequest('GET', f'meva_database/get_linked_tracks?pageSize={limit}')

def get_actor_checkin_photos(gc: girder_client.GirderClient, ):
    data = gc.sendRestRequest('GET', f'meva_database/get_actor_chips')
    results = data['pageResults']
    actor_checkin_map = {}
    for item in results:
        actor = item['actor']
        date = item['date']
        outfit = item['outfit']
        actor_checkin_map[f'{actor}-{date}-{outfit}'] = {
            'back': item['back'],
            'front': item['front']
        }
    return actor_checkin_map

def get_folder_name(gc: girder_client.GirderClient, folderId):
    return gc.get(f'/folder/{folderId}')['name']

def get_link_data(gc: girder_client.GirderClient, folderId, trackId):
    return gc.get(f'meva_detection/meva_chip?folderId={folderId}&trackId={trackId}')

@click.command(name="getActorLinks", help="Creates JSON files for poses/bounds for actor links and starts downloading all relevant images")
@click.argument('limit')
def load_data(limit):

    gc = login()
    actor_links = get_actor_links(gc, limit)['pageResults']
    if os.path.exists('./actors') is False:
        os.mkdir('./actors')
    count = 0
    for actor in actor_links:
        if actor['type'] != 'actor': #filter out track-to-track links for now
            continue
        actorId = actor['actorId']
        date = actor['date']
        outfitId = actor['outfitId']
        links = actor['links']
        print(f'Importing Actor {count} of {len(actor_links)}')
        # create the actorID/date/OutfitId structure
        if os.path.exists(f'./actors/{actorId}') is False:
            os.mkdir(f'./actors/{actorId}/')
        if os.path.exists(f'./actors/{actorId}/{date}') is False:
            os.mkdir(f'./actors/{actorId}/{date}')
        if os.path.exists(f'./actors/{actorId}/{date}/{outfitId}') is False:
            os.mkdir(f'./actors/{actorId}/{date}/{outfitId}')
        # Now that we have the links we take the folderIds get the video names download the json and the images
        root_path = f'./actors/{actorId}/{date}/{outfitId}'
        for link in links:
            if link['type'] != 'track': # only looking for tracks in here
                continue
            folder_name = get_folder_name(gc, link['folderId'])
            if os.path.exists(f'{root_path}/{folder_name}') is False:
                os.mkdir(f'{root_path}/{folder_name}')
            # now we get the link data and download the individual chips
            link_data  = get_link_data(gc, link['folderId'], link['trackId'])
            with open(f'{root_path}/{folder_name}/chips.json', 'w') as outfile:
                json.dump(link_data, outfile)
            for chip in link_data: # download chips and full frames
                if 'chipId' in chip.keys():
                    if os.path.exists( f'{root_path}/{folder_name}/chip_{chip["frame"]}.jpg') is False:
                        gc.downloadFile(chip['chipId'], f'{root_path}/{folder_name}/chip_{chip["frame"]}.jpg')
                        # gc.downloadFile(chip['frameId'], f'{root_path}/{folder_name}/frame_{chip["frame"]}.jpg')

        count = count + 1 


if __name__ == '__main__':
    load_data()

