'''
This script demonstrates CPU descriptor extraction with a reid net found here:
https://github.com/michuanhaohao/reid-strong-baseline/

'''

import sys
import torch 

sys.path.append('/home/local/KHQ/daniel.davila/projects/public_repos/reid-strong-baseline')
from config import cfg as dg_cfg
from modeling import build_model as build_dg_model


config_path = '/home/local/KHQ/daniel.davila/projects/public_repos/reid-strong-baseline/configs/softmax_triplet_with_center.yml'
model_path = '/home/local/KHQ/daniel.davila/market_resnet50_model_120_rank1_945.pth'

# Provision model
dg_cfg.merge_from_file(config_path)
dg_cfg['MODEL']['PRETRAIN_CHOICE'] = None # Prevent model from trying to init backbone
dg_cfg['MODEL']['PRETRAIN_PATH'] = None
dg_model = build_dg_model(dg_cfg, 751)
checkpoint = torch.load(model_path, map_location="cpu")

dg_model.load_state_dict(checkpoint)
_ = dg_model.eval()

# Create model input, size: [batch, channe, width, height]
model_in = torch.ones((1,3,256,128))

with torch.no_grad():
    out = dg_model(model_in)

# >> tensor([[-0.8483, -0.0888, -0.0831,  ..., -1.0392, -0.1856, -0.6668]])

# Benchmark
import time
import numpy as np

times = []
for ii in range(10):
    start = time.time()
    out = dg_model(model_in)
    times.append(time.time() - start)

print(np.mean(times))

# >> 0.114 seconds on a i9-9900K CPU @ 3.60GHz

