import cv2
import numpy as np
import torch
from collections import deque
import itertools
import torchvision

import sys
sys.path.append('/home/local/KHQ/daniel.davila/projects/FairMOT/src/')
import _init_paths
from tracking_utils.utils import *
from tracking_utils.log import logger
from tracking_utils.kalman_filter import KalmanFilter
from tracker import matching
from tracker.basetrack import BaseTrack, TrackState
 

def nms(dets, thresh):
        dets = np.asarray(dets)
        x1 = dets[:, 0]
        y1 = dets[:, 1]
        x2 = dets[:, 2]
        y2 = dets[:, 3]
        scores = dets[:, 4]

        areas = (x2 - x1 + 1) * (y2 - y1 + 1)
        order = scores.argsort()[::-1]

        ndets = len(dets)
        suppressed = np.zeros((ndets), dtype=np.int)

        keep = []
        for _i in range(ndets):
            i = order[_i]
            if suppressed[i] == 1:
                continue
            keep.append(i)
            ix1 = x1[i]
            iy1 = y1[i]
            ix2 = x2[i]
            iy2 = y2[i]
            iarea = areas[i]
            for _j in range(_i, ndets):
                j = order[_j]

                if scores[j] == 1:
                    continue

                if suppressed[j] == 1:
                    continue
                xx1 = max(ix1, x1[j])
                yy1 = max(iy1, y1[j])
                xx2 = min(ix2, x2[j])
                yy2 = min(iy2, y2[j])
                w = max(0.0, xx2 - xx1 + 1)
                h = max(0.0, yy2 - yy1 + 1)
                inter = w * h
                ovr = inter / (iarea + areas[j] - inter)
                if ovr >= thresh:
                    suppressed[j] = 1

        return np.array(keep)

def letterbox(img, height=608, width=1088,
              color=(127.5, 127.5, 127.5)):  # resize a rectangular image to a padded rectangular
    shape = img.shape[:2]  # shape = [height, width]
    ratio = min(float(height) / shape[0], float(width) / shape[1])
    new_shape = (round(shape[1] * ratio), round(shape[0] * ratio))  # new_shape = [width, height]
    dw = (width - new_shape[0]) / 2  # width padding
    dh = (height - new_shape[1]) / 2  # height padding
    top, bottom = round(dh - 0.1), round(dh + 0.1)
    left, right = round(dw - 0.1), round(dw + 0.1)
    img = cv2.resize(img, new_shape, interpolation=cv2.INTER_AREA)  # resized, no border
    img = cv2.copyMakeBorder(img, top, bottom, left, right, cv2.BORDER_CONSTANT, value=color)  # padded rectangular
    return img, ratio, dw, dh

def tensorletterbox(x, height=256, width=256,fillvalue=0):
    shape = x.shape[2:]
    #try:
    ratio = min(float(height) / shape[0], float(width) / shape[1])
    #except:
    #    import xdev
    #    xdev.embed()
    
    new_shape = (round(shape[0] * ratio), round(shape[1] * ratio))
    dw = (width - new_shape[1]) / 2  # width padding
    dh = (height - new_shape[0]) / 2
    top, bottom = round(dh - 0.1), round(dh + 0.1)
    left, right = round(dw - 0.1), round(dw + 0.1)
    x = torch.nn.functional.interpolate(x,size=new_shape,
                                        mode='bilinear')
    x = torch.nn.functional.pad(x, (left,right,top,bottom), mode='constant', value=fillvalue)
    return x

class STrack(BaseTrack):
    shared_kalman = KalmanFilter()
    def __init__(self, tlwh, score, temp_feat, buffer_size=30):

        # wait activate
        self._tlwh = np.asarray(tlwh, dtype=np.float)
        self.most_recent_raw_tlwh = self._tlwh
        self.kalman_filter = None
        self.mean, self.covariance = None, None
        self.is_activated = False

        self.score = score
        self.tracklet_len = 0

        self.smooth_feat = None
        self.update_features(temp_feat)

        # Should the original feat be added to the queue here? Added on top of author code
        self.features = deque([temp_feat / np.linalg.norm(temp_feat)],
                              maxlen=buffer_size)
        self.alpha = 0.9

    def update_features(self, feat):
        feat /= np.linalg.norm(feat)
        self.curr_feat = feat
        if self.smooth_feat is None:
            self.smooth_feat = feat
        else:
            self.smooth_feat = self.alpha * self.smooth_feat + (1 - self.alpha) * feat
        self.features.append(feat)
        self.smooth_feat /= np.linalg.norm(self.smooth_feat)

    def predict(self):
        mean_state = self.mean.copy()
        if self.state != TrackState.Tracked:
            mean_state[7] = 0
        self.mean, self.covariance = self.kalman_filter.predict(mean_state, self.covariance)

    @staticmethod
    def multi_predict(stracks):
        if len(stracks) > 0:
            multi_mean = np.asarray([st.mean.copy() for st in stracks])
            multi_covariance = np.asarray([st.covariance for st in stracks])
            for i, st in enumerate(stracks):
                if st.state != TrackState.Tracked:
                    multi_mean[i][7] = 0
            multi_mean, multi_covariance = STrack.shared_kalman.multi_predict(multi_mean, multi_covariance)
            for i, (mean, cov) in enumerate(zip(multi_mean, multi_covariance)):
                stracks[i].mean = mean
                stracks[i].covariance = cov

    def activate(self, kalman_filter, frame_id):
        """Start a new tracklet"""
        self.kalman_filter = kalman_filter
        self.track_id = self.next_id()
        self.mean, self.covariance = self.kalman_filter.initiate(self.tlwh_to_xyah(self._tlwh))

        self.tracklet_len = 0
        self.state = TrackState.Tracked
        if frame_id == 1:
            self.is_activated = True
        #self.is_activated = True
        self.frame_id = frame_id
        self.start_frame = frame_id

    def re_activate(self, new_track, frame_id, new_id=False):
        self.mean, self.covariance = self.kalman_filter.update(
            self.mean, self.covariance, self.tlwh_to_xyah(new_track.tlwh)
        )

        #if int(new_track.tlwh[0]) == 1091 and int(new_track.tlwh[1]) == 170:
        #    print('Found the problem box')
        #    import xdev
        #    xdev.embed()
        self.most_recent_raw_tlwh = new_track.tlwh

        self.update_features(new_track.curr_feat)
        self.tracklet_len = 0
        self.state = TrackState.Tracked
        self.is_activated = True
        self.frame_id = frame_id
        self.score = new_track.score
        if new_id:
            self.track_id = self.next_id()

    def update(self, new_track, frame_id, update_feature=True):
        """
        Update a matched track
        :type new_track: STrack
        :type frame_id: int
        :type update_feature: bool
        :return:
        """
        self.frame_id = frame_id
        self.tracklet_len += 1

        new_tlwh = new_track.tlwh
        self.most_recent_raw_tlwh = new_tlwh
        self.mean, self.covariance = self.kalman_filter.update(
            self.mean, self.covariance, self.tlwh_to_xyah(new_tlwh))
        self.state = TrackState.Tracked
        self.is_activated = True

        self.score = new_track.score
        if update_feature:
            self.update_features(new_track.curr_feat)

    @property
    # @jit(nopython=True)
    def tlwh(self):
        """Get current position in bounding box format `(top left x, top left y,
                width, height)`.
        """
        if self.mean is None:
            return self._tlwh.copy()
        ret = self.mean[:4].copy()
        ret[2] *= ret[3]
        ret[:2] -= ret[2:] / 2
        return ret

    @property
    # @jit(nopython=True)
    def tlbr(self):
        """Convert bounding box to format `(min x, min y, max x, max y)`, i.e.,
        `(top left, bottom right)`.
        """
        ret = self.tlwh.copy()
        ret[2:] += ret[:2]
        return ret

    @staticmethod
    # @jit(nopython=True)
    def tlwh_to_xyah(tlwh):
        """Convert bounding box to format `(center x, center y, aspect ratio,
        height)`, where the aspect ratio is `width / height`.
        """
        ret = np.asarray(tlwh).copy()
        ret[:2] += ret[2:] / 2
        ret[2] /= ret[3]
        return ret

    def to_xyah(self):
        return self.tlwh_to_xyah(self.tlwh)

    @staticmethod
    # @jit(nopython=True)
    def tlbr_to_tlwh(tlbr):
        ret = np.asarray(tlbr).copy()
        ret[2:] -= ret[:2]
        return ret

    @staticmethod
    # @jit(nopython=True)
    def tlwh_to_tlbr(tlwh):
        ret = np.asarray(tlwh).copy()
        ret[2:] += ret[:2]
        return ret

    def __repr__(self):
        return 'OT_{}_({}-{})'.format(self.track_id, self.start_frame, self.end_frame)

def joint_stracks(tlista, tlistb):
    exists = {}
    res = []
    for t in tlista:
        exists[t.track_id] = 1
        res.append(t)
    for t in tlistb:
        tid = t.track_id
        if not exists.get(tid, 0):
            exists[tid] = 1
            res.append(t)
    return res


def sub_stracks(tlista, tlistb):
    stracks = {}
    for t in tlista:
        stracks[t.track_id] = t
    for t in tlistb:
        tid = t.track_id
        if stracks.get(tid, 0):
            del stracks[tid]
    return list(stracks.values())


def remove_duplicate_stracks(stracksa, stracksb):
    pdist = matching.iou_distance(stracksa, stracksb)
    pairs = np.where(pdist < 0.15)
    dupa, dupb = list(), list()
    for p, q in zip(*pairs):
        timep = stracksa[p].frame_id - stracksa[p].start_frame
        timeq = stracksb[q].frame_id - stracksb[q].start_frame
        if timep > timeq:
            dupb.append(q)
        else:
            dupa.append(p)
    resa = [t for i, t in enumerate(stracksa) if not i in dupa]
    resb = [t for i, t in enumerate(stracksb) if not i in dupb]
    return resa, resb


class JDETracker(object):
    def __init__(self, opt, 
            det_config_file=None,
            det_weights_path=None,
            dg_model_weights_path=None,
            frame_rate=30):

        import torch
        import sys
        
        sys.path.append('/opt/mevaTracker/vendor/CenterNet2/projects/CenterNet2/')
        from detectron2.modeling import build_model as build_det_model
        from detectron2.config import get_cfg as get_det_cfg
        from centernet.config import add_centernet_config
        from detectron2.checkpoint import DetectionCheckpointer

        assert det_config_file is not None, "ERROR: Must configure det_config_file"
        assert det_weights_path is not None, "ERROR: Must configure det_weights_path"
        assert dg_model_weights_path is not None, "ERROR: Must configure dg_model_weights_path"

        self.opt = opt
        if opt.gpus[0] >= 0:
            opt.device = torch.device('cuda')
        else:
            opt.device = torch.device('cpu')

        print('Creating models...')
                
        #Custom detector model ##########################################

        '''
        config_file ='/home/local/KHQ/daniel.davila/projects/public_repos/CenterNet2/projects/CenterNet2/configs/CenterNet2_DLA-BiFPN-P3_4x_ursa.yaml'
        det_cfg = get_det_cfg()
        add_centernet_config(det_cfg)
        det_cfg.merge_from_file(config_file)
        det_cfg.MODEL.DEVICE='cuda'
        det_cfg.freeze()

        checkpoint = torch.load("/home/local/KHQ/daniel.davila/CenterNet2_DLA-BiFPN-P3_4x.pth")
        del checkpoint['model']['pixel_std']
        del checkpoint['model']['pixel_mean']
        model = build_det_model(det_cfg)
        '''

        config_file = det_config_file
        #config_file ='/home/local/KHQ/daniel.davila/projects/public_repos/CenterNet2/projects/CenterNet2/configs/CenterNet2_DLA-BiFPN-P3_4x_ursa.yaml'
        cfg = get_det_cfg()
        add_centernet_config(cfg)
        cfg.merge_from_file(config_file)
        cfg.MODEL.DEVICE='cuda'
        cfg['MODEL']['WEIGHTS'] = ''
        cfg['MODEL']['CENTERNET']['NMS_TH_TEST'] = 1.0
        cfg['MODEL']['CENTERNET']['NMS_TH_TRAIN'] = 1.0
        cfg['MODEL']['CENTERNET']['POST_NMS_TOPK_TEST'] = 1000
        cfg.freeze()

        checkpoint = torch.load(det_weights_path)
        #checkpoint = torch.load("/home/local/KHQ/daniel.davila/CenterNet2_DLA-BiFPN-P3_4x.pth")
        del checkpoint['model']['pixel_std']
        del checkpoint['model']['pixel_mean']
        
        #import xdev
        #xdev.embed()
        model = build_det_model(cfg)
        model = model.eval()
        model.load_state_dict(checkpoint['model'])
        self.model = model
        #################################################################
       
        #Custom descriptor gen model ####################################
        
        #TODO change this path to match the container mount point for this model
        #TODO dont yell at daniel.davila for this messy hack
        #sys.path.append('/home/local/KHQ/daniel.davila/projects/public_repos/reid-strong-baseline/')
        sys.path.append('/opt/mevaTracker/vendor/reid-strong-baseline/')
        from config import cfg as dg_cfg
        from modeling import build_model as build_dg_model

        dg_cfg.merge_from_file('/opt/mevaTracker/artifacts/softmax_triplet_with_center.yml')
        dg_cfg['MODEL']['PRETRAIN_CHOICE'] = None # Prevent model from trying to init backbone
        dg_cfg['MODEL']['PRETRAIN_PATH'] = None
        dg_model = build_dg_model(dg_cfg, 751)
        self.dg_cfg = dg_cfg


        checkpoint = torch.load(dg_model_weights_path, map_location="cpu")
        #checkpoint = torch.load('/home/local/KHQ/daniel.davila/market_resnet50_model_120_rank1_945.pth',
                                #dg_cfg.MODEL.PRETRAIN_PATH,
                                # In-case weights were saved with the
                                # context of a non-CPU device.
        #                        map_location="cpu")

        dg_model.load_state_dict(checkpoint)
        _ = dg_model.eval().cuda()
        self.dg_model = dg_model
        
        #################################################################
    
        self.tracked_stracks = []  # type: list[STrack]
        self.lost_stracks = []  # type: list[STrack]
        self.removed_stracks = []  # type: list[STrack]

        self.frame_id = 0
        self.det_thresh = opt.conf_thres
        self.buffer_size = int(frame_rate / 30.0 * opt.track_buffer)
        self.max_time_lost = self.buffer_size
        self.max_per_image = opt.K
        self.mean = np.array(opt.mean, dtype=np.float32).reshape(1, 1, 3)
        self.std = np.array(opt.std, dtype=np.float32).reshape(1, 1, 3)

        self.kalman_filter = KalmanFilter()

    def post_process(self, dets, meta):
        dets = dets.detach().cpu().numpy()
        dets = dets.reshape(1, -1, dets.shape[2])
        dets = ctdet_post_process(
            dets.copy(), [meta['c']], [meta['s']],
            meta['out_height'], meta['out_width'], self.opt.num_classes)
        for j in range(1, self.opt.num_classes + 1):
            dets[0][j] = np.array(dets[0][j], dtype=np.float32).reshape(-1, 5)
        return dets[0]

    def merge_outputs(self, detections):
        results = {}
        for j in range(1, self.opt.num_classes + 1):
            results[j] = np.concatenate(
                [detection[j] for detection in detections], axis=0).astype(np.float32)

        scores = np.hstack(
            [results[j][:, 4] for j in range(1, self.opt.num_classes + 1)])
        if len(scores) > self.max_per_image:
            kth = len(scores) - self.max_per_image
            thresh = np.partition(scores, kth)[kth]
            for j in range(1, self.opt.num_classes + 1):
                keep_inds = (results[j][:, 4] >= thresh)
                results[j] = results[j][keep_inds]
        return results

    def update(self, im_blob, priors=None):
        '''
        Inputs:
            im_blob (tensor) : A torch.tensor with shape shape(1, channels, height, width). Image should be normalized to [0,1].
            prios (dict) : Optional. Dictionary of prior tracks to fuse into this tracking result.
        '''


        self.frame_id += 1
        activated_starcks = []
        refind_stracks = []
        lost_stracks = []
        removed_stracks = []


        ''' Step 1: Network forward, get detections & embeddings'''
        with torch.no_grad():
            model_in = torch.tensor(im_blob[0] * 255.0).cuda()
            out = self.model.forward([{'image' : model_in}])
            bboxes = out[0]['instances'].get('pred_boxes').tensor.cpu().numpy()
            scores = out[0]['instances'].get('scores').cpu().numpy()
            classes = out[0]['instances'].get('pred_classes').cpu().numpy()
            
            '''output = self.model(im_blob)[-1]
            hm = output['hm'].sigmoid_()
            wh = output['wh']
            id_feature = output['id']
            id_feature = F.normalize(id_feature, dim=1)

            reg = output['reg'] if self.opt.reg_offset else None
            dets, inds = mot_decode(hm, wh, reg=reg, ltrb=self.opt.ltrb, K=self.opt.K)
            id_feature = _tranpose_and_gather_feat(id_feature, inds)
            id_feature = id_feature.squeeze(0)
            id_feature = id_feature.cpu().numpy()'''

        dets = np.concatenate([bboxes, scores[:,np.newaxis]], axis=1)
        person_inds = classes == 0 # Person ID in COCO TODO make configurable
        dets = dets[person_inds]
        remain_inds = dets[:, 4] > self.opt.conf_thres
        dets = dets[remain_inds]

        # TODO does this break if dets empty?
        if priors is not None:
            for pdets in priors:
                x1,y1,x2,y2 = pdets['geom']['g0']
                #Ensure that bboxes are valid
                if x2>x1 and y2>y1 and x1>0 and x2>0 and y1>0 and y2>0:
                    dets = np.concatenate([dets, [[x1,y1,x2,y2,1.0],]], axis=0)

        if len(dets):
            keeps = nms(dets, 0.5)

            # Make sure we lost no priors
            if priors is not None:
                assert sum(dets[:,4] == 1) == sum(dets[keeps][:,4] == 1)

            dets = dets[keeps]
        
            # Extract descriptor for each bbox
            feats = []
            with torch.no_grad():
                model_in = torch.zeros((len(dets), 3, 256, 128)).cuda()
                for ii, det in enumerate(dets):
                    x1, y1, x2, y2 = [int(a) for a in det[:4]]  
                    chip = im_blob[:,:,y1:y2,x1:x2]
                    chip = torchvision.transforms.functional.normalize(chip, self.dg_cfg.INPUT.PIXEL_MEAN, self.dg_cfg.INPUT.PIXEL_STD)
                    if np.any(np.array(chip.shape) <= 0): # and chip.shape[3] <= 0:
                        print('Found invalid size chip', chip.shape)
                        chip = torch.zeros((1,1,256,128), dtype=torch.float)
                    else:
                        chip = tensorletterbox(chip, height=256, width=128)
                    model_in[ii,:,:,:] = chip[0,:,:,:]
                try:
                    dg = self.dg_model(model_in)
                except:
                    import xdev
                    xdev.embed()
                id_feature = dg.cpu().numpy()
                #feats.append(dg.cpu().numpy())

            '''Detections'''
            #id_feature = np.concatenate(feats)
            detections = [STrack(STrack.tlbr_to_tlwh(tlbrs[:4]), tlbrs[4], f, 30) for
                          (tlbrs, f) in zip(dets[:, :5], id_feature)]
        else:
            if_feature = []
            detections = []

        ''' Add newly detected tracklets to tracked_stracks'''
        unconfirmed = []
        tracked_stracks = []  # type: list[STrack]
        for track in self.tracked_stracks:
            if not track.is_activated:
                unconfirmed.append(track)
            else:
                tracked_stracks.append(track)

                
        def embedding_distance(tracks, detections, metric='cosine'):
            """
            :param tracks: list[STrack]
            :param detections: list[BaseTrack]
            :param metric:
            :return: cost_matrix np.ndarray
            """

            from scipy.spatial.distance import cdist
            
            cost_matrix = np.zeros((len(tracks), len(detections)), dtype=np.float)
            if cost_matrix.size == 0:
                return cost_matrix
            
            det_features = np.asarray([track.curr_feat for track in detections], dtype=np.float) 
            #(dets, 2048)
            #track_features = np.asarray([track.smooth_feat for track in tracks], dtype=np.float)
            # (tracks, 2048)
            
            dists = []
            for track in tracks:
                #extract dist between each det feat and each feat of current track history
                track_dists = np.min(cdist(det_features, np.array(track.features)[-30:], metric), axis=1) #(dets, <=10)
                dists.append(track_dists)

            cost_matrix = np.maximum(0.0, np.vstack(dists))  # Nomalized features
            
            #cost_matrix = np.maximum(0.0, cdist(track_features, det_features, metric))  # Nomalized features
            return cost_matrix
                
                
        ''' Step 2: First association, with embedding'''
        strack_pool = joint_stracks(tracked_stracks, self.lost_stracks)
        # Predict the current location with KF
        #for strack in strack_pool:
            #strack.predict()
        STrack.multi_predict(strack_pool)
        dists = matching.embedding_distance(strack_pool, detections)
        #dists = embedding_distance(strack_pool, detections)

        #dists = matching.iou_distance(strack_pool, detections)
        dists = matching.fuse_motion(self.kalman_filter, dists, strack_pool, detections)
        matches, u_track, u_detection = matching.linear_assignment(dists, thresh=0.4)

        for itracked, idet in matches:
            track = strack_pool[itracked]
            det = detections[idet]
            if track.state == TrackState.Tracked:
                track.update(detections[idet], self.frame_id)
                activated_starcks.append(track)
            else:
                track.re_activate(det, self.frame_id, new_id=False)
                refind_stracks.append(track)

        ''' Step 3: Second association, with IOU'''
        detections = [detections[i] for i in u_detection]
        r_tracked_stracks = [strack_pool[i] for i in u_track if strack_pool[i].state == TrackState.Tracked]
        dists = matching.iou_distance(r_tracked_stracks, detections)
        matches, u_track, u_detection = matching.linear_assignment(dists, thresh=0.5)

        for itracked, idet in matches:
            track = r_tracked_stracks[itracked]
            det = detections[idet]
            if track.state == TrackState.Tracked:
                track.update(det, self.frame_id)
                activated_starcks.append(track)
            else:
                track.re_activate(det, self.frame_id, new_id=False)
                refind_stracks.append(track)
                
        for it in u_track:
            track = r_tracked_stracks[it]
            if not track.state == TrackState.Lost:
                track.mark_lost()
                lost_stracks.append(track)

        '''Deal with unconfirmed tracks, usually tracks with only one beginning frame'''
        detections = [detections[i] for i in u_detection]
        dists = matching.iou_distance(unconfirmed, detections)
        matches, u_unconfirmed, u_detection = matching.linear_assignment(dists, thresh=0.7)
        for itracked, idet in matches:
            unconfirmed[itracked].update(detections[idet], self.frame_id)
            activated_starcks.append(unconfirmed[itracked])
        for it in u_unconfirmed:
            track = unconfirmed[it]
            track.mark_removed()
            removed_stracks.append(track)

        """ Step 4: Init new stracks"""
        for inew in u_detection:
            track = detections[inew]
            if track.score < self.det_thresh:
                continue
            track.activate(self.kalman_filter, self.frame_id)
            activated_starcks.append(track)
        """ Step 5: Update state"""
        for track in self.lost_stracks:
            if self.frame_id - track.end_frame > self.max_time_lost:
                track.mark_removed()
                removed_stracks.append(track)

        # print('Ramained match {} s'.format(t4-t3))

        self.tracked_stracks = [t for t in self.tracked_stracks if t.state == TrackState.Tracked]
        self.tracked_stracks = joint_stracks(self.tracked_stracks, activated_starcks)
        self.tracked_stracks = joint_stracks(self.tracked_stracks, refind_stracks)
        self.lost_stracks = sub_stracks(self.lost_stracks, self.tracked_stracks)
        self.lost_stracks.extend(lost_stracks)
        self.lost_stracks = sub_stracks(self.lost_stracks, self.removed_stracks)
        self.removed_stracks.extend(removed_stracks)
        self.tracked_stracks, self.lost_stracks = remove_duplicate_stracks(self.tracked_stracks, self.lost_stracks)
        # get scores of lost tracks
        output_stracks = [track for track in self.tracked_stracks if track.is_activated]

        logger.debug('===========Frame {}=========='.format(self.frame_id))
        logger.debug('Activated: {}'.format([track.track_id for track in activated_starcks]))
        logger.debug('Refind: {}'.format([track.track_id for track in refind_stracks]))
        logger.debug('Lost: {}'.format([track.track_id for track in lost_stracks]))
        logger.debug('Removed: {}'.format([track.track_id for track in removed_stracks]))

        return output_stracks

