import girder_client
import json
import click

'''
python3 MEVAS3Import.py --data JSONFile.json
Json file has 3 root keys:

apiURL:  base URL of the MEVA Server
folderId: The folderId to upload the data into

data: Array of objects containing the information that is needed to upload
serverURL: URL pointing to the file that should be downloaded
apiKey: if using an apikey instead of being prompted for credentials
folderId: base folder to upload the data contents to
 data: [
    {
        filename: filename to use for the downloaded file
        tracksURL: Optional trackURL file
        descriptorURL: Optional descriptor file for the dataset
    },
 ]

'''

@click.command(name="Server Upload", help="Uses a JSON format to upload data to a specific MEVA Folder")
@click.argument('data')
def load_data(data):
    f = open(data, 'r')
    json_string = f.read()
    data_obj = json.loads(json_string)
    apiKey = None
    apiURL = data_obj['serverURL']
    gc = girder_client.GirderClient(apiURL, apiRoot='girder/api/v1')
    if 'apiKey' in data_obj:
        apiKey = data_obj['apiKey']
        gc.authenticate(apiKey=apiKey)
    else:
        print(f"Enter Girder Credentials for {apiURL}")
        gc.authenticate(interactive=True)
    folderId = data_obj['folderId']
    data_list = data_obj['data']

    folderList = []
    for item in data_list:
        if 'videoURL' not in item:
            raise Exception('You need to include a videoURL')
        if 'filename' not in item:
            raise Exception('You need to include a filename')
        folder = {
            'videoURL': item['videoURL'],
            'filename': item['filename'],
        }
        if 'tracksURL' in item:
            folder['tracksURL'] = item['tracksURL']
        if 'descriptorURL' in item:
            folder['descriptorURL'] = item['descriptorURL']
        folderList.append(folder)

    gc.sendRestRequest('PUT', f"meva/s3_import/{folderId}", json=folderList)

if __name__ == '__main__':
    load_data()
