#! /usr/bin/python3.7
"""
Copyright 2021 by Kitware, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Kitware, Inc. nor the names of any contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

==============================================================================

"""
from __future__ import division, print_function
import os
import sys
import numpy as np
import argparse
import pytz
import logging
from pathlib import Path
from meva_server.meva_database.database import MevaDatabase
from meva_server.meva_database.models import Track, Source


if __name__ == "__main__":
    print("Reading arguments...")
    # parse the arguments - required for location of database
    parser = argparse.ArgumentParser(
        description="Script to add gps ids to exemplar tracks"
    )
    parser.add_argument("databasePath", help="path to database file")

    args = parser.parse_args()
    if args.databasePath == "":
        print("No databasePath specified - exiting.")
        sys.exit()

    # connect to the db
    print("Setting up db...")
    database_path = Path(args.databasePath)

    os.environ["MEVA_DATABASE_URI"] = "sqlite:///" + str(database_path)
    meva_db = MevaDatabase()
    session = meva_db.Session()

    # turn off sqlalchemy logs
    for name in logging.root.manager.loggerDict:
        if "sqlalchemy" in name:
            logger = logging.getLogger(name)
            logger.disabled = True

    print('Loading all tracks...')
    tracks = session.query(Track).all()
    print('Loading all sources...')
    sources = session.query(Source).all()

    print('removing non exemplar tracks')
    eTracks = []
    for track in tracks:
        if track.exemplar:
            eTracks.append(track)
    print(f"There are {len(eTracks)} exemplar tracks")

    print('adding in gps ids')
    # loop though all the tracks and get the gpsid
    for track in eTracks:
        for src in sources:
            if track.source_uid == src.uid:
                track.gps_ids = 'G' + str(src.gps_tracker_id)
                track.exemplar_image1 = src.name
                #print(src.gps_tracker_id)

    session.commit()
