'''
Pulls link chips from an imported set (see the 'GetLinkedChip.py' script) and
creates a dataset folder with the chips labeled like this:
i.e. 082_G341_20180307_003_5817
082 = person ID
G341 = camera ID
20180307 = day
003 = outfit ID
5817 = frame number
'''
import json
import os
import glob
import pathlib
from shutil import copyfile

from numpy.core.numeric import full

# folder containing the export from MEVA
linkFolder = './links/'  # this should have an 'actors' folder from the GetLinkedChip.py
outputFolder = './mevid_chips/'
_useForCurrentReID = True  # this will combine the outfit plus the person ID to make unique sets

if __name__ == '__main__':

    if not os.path.exists(outputFolder):
        os.mkdir(outputFolder)

    chipFiles = glob.glob(linkFolder + "actors/**/**/**/*.json",
                          recursive=True)

    #chipFiles = chipFiles[:6]  # temp limit for testing
    matches = {}
    for chipFile in chipFiles:
        bestScore = 0

        chipList = []
        frames = []
        with open(chipFile, 'r') as cf:
            chipList = json.load(cf)
        if len(chipList) > 0:
            for chip in chipList:
                if 'score' in chip.keys():
                    if chip['score'] > bestScore:
                        bestScore = chip['score']
                        frames.append(chip['frame'])
                else:
                    print('ERROR: no score')
        else:
            print('ERROR: empty json chip list')

        for frame in frames:
            pathPar = str(pathlib.Path(chipFile).parent)
            filePath = pathPar + '/chip_' + str(frame) + '.jpg'
            sepFile = filePath.split('/')
            #prefix = sepFile[-1][5:9]
            video = sepFile[5]
            camera = video.split('.')[4]
            actor = sepFile[2]
            dat = sepFile[3]
            outfit = sepFile[4]
            key = actor + '_' + camera + '_' + dat + '_' + outfit + '_' + str(frame)
            if _useForCurrentReID:
                # using the person ID + the outfit for current SOTA
                actor = str( int(actor)*100 + int(outfit) )
                key = actor.zfill(8) + '_' + dat[4:].zfill(6) + '_' + str(frame).zfill(6)
            if key in matches.keys():
                if matches[key]['score'] < bestScore:
                    matches[key]['score'] = bestScore
                    matches[key]['file'] = filePath
            else:
                matches[key] = {}
                matches[key]['score'] = bestScore
                matches[key]['file'] = filePath

    # rename the files to this other format and copy them
    for keyName in matches:
        newName = outputFolder + keyName + '.jpg'
        oldFile = matches[keyName]['file']
        print(f"{oldFile} -> {newName}")
        try:
            copyfile(oldFile, newName)
        except Exception as Ex:
            print(Ex)

    # remove known bad files
    badFrames = [
        '204_G341_20180307_004_3570.jpg'
    ]
    for frame in badFrames:
        os.remove(outputFolder + frame)