#! /usr/bin/python3.7
"""
Copyright 2021 by Kitware, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Kitware, Inc. nor the names of any contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

==============================================================================

"""
from __future__ import division, print_function
import os
import argparse
import json
from pathlib import Path
from meva_server.meva_database.database import MevaDatabase
from meva_server.meva_database.models import Track, Source, Response
from meva_server.meva_database.utils import turnOffSqlAlchemyLogging

if __name__ == "__main__":

    print('Reading arguments...')
    # parse the arguments - required for location of database
    parser = argparse.ArgumentParser(
        description='Script to see how many links are confirmed')
    parser.add_argument('databasePath', help='path to database file')
    args = parser.parse_args()

    # connect to the db
    print('Setting up db...')
    database_path = Path(args.databasePath)
    os.environ['MEVA_DATABASE_URI'] = 'sqlite:///' + str(database_path)
    meva_db = MevaDatabase()
    session = meva_db.Session()

    turnOffSqlAlchemyLogging()

    # grab all tracks
    print('Loading all tracks...')
    allTracks = session.query(Track).all()
    print('Loading responses...')
    responses = session.query(Response).all()
    print('Loading sources...')
    sources = session.query(Source).all()

    print('Sorting source by uid...')
    sourceByUid = {}
    for src in sources:
        sourceByUid[src.uid] = src

    print('Loading exemplars...')
    exemplars = {}
    tracks = {}
    for track in allTracks:
        # make the track easily referenced by uid
        tracks[track.uid] = track
        if track.exemplar:
            exemplars[track.uid] = track

    # sort the responses
    print('Sorting responses...')
    yeses = []
    nos = []
    idks = []
    for resp in responses:
        if resp.response == 1:
            yeses.append(resp)
        elif resp.response == 0:
            nos.append(resp)
        else:
            idks.append(resp)

    print(f"Yes responses: {len(yeses)}")
    print(f"No responses: {len(nos)}")
    print(f"IDK responses: {len(idks)}")

    # loop through the yeses and get the details
    positiveLinks = {}
    cnt = 0
    for yes in yeses:
        src = {}
        link = {}
        t1 = yes.track1_uid
        t2 = yes.track2_uid
        src1 = sourceByUid[tracks[t1].source_uid]
        src2 = sourceByUid[tracks[t2].source_uid]
        if t1 in exemplars:
            # t1 is actor
            src['type'] = 'actor'
            src['actorId'] = src1.girder_id
            src['date'] = src1.name.split('-')[1]
            src['outfitId'] = src1.outfit_id
            # next must be video
            link['folderId'] = src2.uid
            link['trackId'] = tracks[t2].video_track_id
            link['type'] = 'track'
        elif t2 in exemplars:
            # t2 is actor
            src['type'] = 'actor'
            src['actorId'] = src2.girder_id
            src['date'] = src2.name.split('-')[1]
            src['outfitId'] = src2.outfit_id
            # next must be video
            link['folderId'] = src1.uid
            link['trackId'] = tracks[t1].video_track_id
            link['type'] = 'track'
        else:
            # neither are actors
            src['folderId'] = src1.uid
            src['trackId'] = tracks[t1].video_track_id
            src['type'] = 'track'
            link['folderId'] = src2.uid
            link['trackId'] = tracks[t2].video_track_id
            link['type'] = 'track'

        positiveLinks[cnt] = {}
        positiveLinks[cnt]['source'] = src
        positiveLinks[cnt]['link'] = link
        cnt += 1

    #print(json.dumps(positiveLinks, indent=4, sort_keys=True))

    session.commit()
