import os
import numpy as np
import cv2
import fnmatch
import matplotlib.pyplot as plt
import shutil
from random import shuffle

check_in_base_dir0 = '/home/local/KHQ/matt.brown/diva_checkin_photos/raw_images_march'
check_in_base_dir = '/home/local/KHQ/matt.brown/diva_checkin_photos/reID_actor_dossier/MARCH_reID_actor_chips'
out_dir = '/home/local/KHQ/matt.brown/diva_checkin_photos/reID_actor_dossier/matt_renamed'
out_dir_sanity = '/home/local/KHQ/matt.brown/diva_checkin_photos/reID_actor_dossier/matt_renamed_sanity'


img_fnames0 = []
for dirpath, dirnames, files in os.walk(check_in_base_dir0):
    for img_ext in ['*.jpg', '*.JPG', '*.png']:
        img_fnames0 = img_fnames0 + ['%s/%s' % (dirpath, fname)
                                     for fname in fnmatch.filter(files, img_ext)]

img_fnames0 = sorted(img_fnames0)

img_fnames = []
for dirpath, dirnames, files in os.walk(check_in_base_dir):
    for img_ext in ['*.jpg', '*.JPG', '*.png']:
        img_fnames = img_fnames + ['%s/%s' % (dirpath, fname)
                                   for fname in fnmatch.filter(files, img_ext)]

#img_fnames = sorted(img_fnames)
shuffle(img_fnames)


def downsample(image):
    return cv2.pyrDown(cv2.pyrDown(cv2.pyrDown(image)))


def match_images_orb(imgi, imgj):
    orb = cv2.ORB_create(nfeatures=10000, edgeThreshold=21,
                         patchSize=31, nlevels=16,
                         scoreType=cv2.ORB_FAST_SCORE, fastThreshold=10)

    if imgi.ndim == 3:
        imgi_gray = cv2.cvtColor(imgi, cv2.COLOR_RGB2GRAY)
    else:
        imgi_gray = imgi

    if imgj.ndim == 3:
        imgj_gray = cv2.cvtColor(imgj, cv2.COLOR_RGB2GRAY)
    else:
        imgj_gray = imgj

    kp0, des0 = orb.detectAndCompute(imgi_gray, None)
    kp1, des1 = orb.detectAndCompute(imgj_gray, None)

    bf = cv2.BFMatcher(cv2.NORM_HAMMING, crossCheck=True)
    matches = bf.match(des0, des1)
    pts0 = np.float32([kp0[m.queryIdx].pt for m in matches])
    pts1 = np.float32([kp1[m.trainIdx].pt for m in matches])
    mask = np.zeros(len(pts0))
    h, mask = cv2.findHomography(pts1.reshape(-1, 1, 2),
                                 pts0.reshape(-1, 1, 2),
                                 method=cv2.RANSAC,
                                 ransacReprojThreshold=2,
                                 mask=mask)
    img_rect = cv2.warpPerspective(imgj, h, imgi.shape[:2][::-1])

    return cv2.matchTemplate(imgi, img_rect, cv2.TM_CCOEFF_NORMED).max()


imagesj = []
for j in range(len(img_fnames0)):
    print(j + 1, 'of', len(img_fnames0))
    imagesj.append(downsample(cv2.imread(img_fnames0[j])))


for i in range(len(img_fnames)):
    print(i + 1, 'of', len(img_fnames))

    if not os.path.isfile(img_fnames[i]):
        print('Cropped file no longer there')
        continue

    scores = np.zeros(len(img_fnames0))
    imgi = downsample(cv2.imread(img_fnames[i]))
    for j in range(len(imagesj)):
        imgj = imagesj[j]

        # Apply template Matching
        try:
            score = cv2.matchTemplate(imgi, imgj, cv2.TM_CCOEFF_NORMED).max()
            #score = match_images_orb(imgi, imgj)
        except:
            continue

        scores[j] = score
        if scores[j] > 0.99:
            #raise Exception()
            break

    j = np.argmax(scores)
    if scores[j] > 0.95:
        # Move the image.
        date = os.path.split(img_fnames0[j])[1].split('_')[0]
        fname, ext = os.path.splitext(os.path.split(img_fnames[i])[1])
        fname_split = fname.split('-')
        if len(fname_split) == 4:
            eid, date0, outfit, facing = fname_split
            #assert date0 == date
        else:
           eid, outfit, facing = fname_split

        fname = '-'.join([eid, date, outfit, facing])

        if True:
            plt.close('all')
            plt.figure(figsize=(18, 12), dpi=80)
            plt.subplot(1,2,1)
            plt.imshow(imgi[:, :, ::-1])
            plt.title(os.path.split(img_fnames[i])[1])
            plt.axis('off')

            imgj = imagesj[j].copy()
            res = cv2.matchTemplate(imgi, imgj, cv2.TM_CCOEFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
            top_left = max_loc
            bottom_right = (top_left[0] + imgi.shape[1], top_left[1] + imgi.shape[0])
            cv2.rectangle(imgj, top_left, bottom_right, 255, 2)
            plt.subplot(1,2,2)
            plt.imshow(imgj[:, :, ::-1])
            plt.title(os.path.split(img_fnames0[j])[1])
            plt.axis('off')

            plt.savefig('%s/%s.jpg' % (out_dir_sanity, fname))
            plt.close('all')

        fname_out = '%s/%s%s' % (out_dir, fname, ext)
        k = 0
        while os.path.isfile(fname_out):
            fname_out = '%s/%s_DUPLICATE%s%s' % (out_dir, fname,
                                                 str(k).zfill(3), ext)
            k += 1

        print('Score', scores[j], 'Moving', fname_out)
        try:
            shutil.move(img_fnames[i], fname_out)
        except FileNotFoundError:
            pass
    else:
        raise Exception()
        continue
