"""
This small script is meant to migrate a db that has already been created
"""
from __future__ import division, print_function
from meva_server.meva_database.utils import turnOffSqlAlchemyLogging
from meva_server.meva_database.models import Track, Response
from meva_server.meva_database.database import MevaDatabase
import os
from pathlib import Path
import sqlalchemy
import time
import argparse

_data_path = '/home/local/KHQ/joseph.vanpelt/projects/meva/'
_db_name = 'meva_from_server.db'


def add_column(session, table_name, column):
    column_name = column.name
    column_type = column.type
    try:
        session.execute('ALTER TABLE %s ADD COLUMN %s %s' %
                        (table_name, column_name, column_type))
    except Exception as msg:
        print('Unable to add column to table:\n' +
              msg)


if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        description='Script to determine which tracks should not be linked')
    parser.add_argument('databasePath', help='path to database file',
                        default=_data_path + _db_name)
    args = parser.parse_args()

    database_path = Path(args.databasePath)
    os.environ['MEVA_DATABASE_URI'] = 'sqlite:///' + str(database_path)
    meva_db = MevaDatabase()
    session = meva_db.Session()

    turnOffSqlAlchemyLogging()

    table_name = 'responses'

    # adjust the table to have some new columns
    column = sqlalchemy.Column(
        'start_time', sqlalchemy.Float, primary_key=False)
    add_column(session, table_name, column)
    # add a column for when the response was saved
    column = sqlalchemy.Column(
        'saved_time', sqlalchemy.Float, primary_key=False)
    add_column(session, table_name, column)

    # populate the saved_time with the current time
    responses = session.query(Response).all()
    for response in responses:
        response.saved_time = time.time()

    # take the start time from first track and add it for day searching
    for response in responses:
        track = session.query(Track).\
            filter(Track.uid == response.track1_uid).first()
        response.start_time = track.start_time

    session.commit()
