'''
This script sets up CPU descriptor extraction with a reid net found here:
https://github.com/michuanhaohao/reid-strong-baseline/
and saves a set of descriptors for actor checkin photos
'''
import sys
# settings (here is the main path used for others below)
data_path = '/home/local/KHQ/joseph.vanpelt/projects/meva/'

# use the code from cloning the above repo
sys.path.append(data_path + 'reid/reid-strong-baseline')
from modeling import build_model as build_dg_model
from config import cfg as dg_cfg
import numpy as np
import glob
import torch
from PIL import Image
import matplotlib.pyplot as plt
from torchvision.transforms import ToTensor, ToPILImage
from torchvision.transforms.functional import normalize
import torch.nn.functional as F
import pathlib
import pickle
from people_segmentation.pre_trained_models import create_model
from iglovikov_helper_functions.dl.pytorch.utils import tensor_from_rgb_image
from iglovikov_helper_functions.utils.image_utils import load_rgb, pad, unpad
import albumentations as albu

def tensorletterbox(x, height=256, width=256, fillvalue=0):
    shape = x.shape[2:]
    #try:
    ratio = min(float(height) / shape[0], float(width) / shape[1])
    #except:
    #    import xdev
    #    xdev.embed()

    new_shape = (round(shape[0] * ratio), round(shape[1] * ratio))
    dw = (width - new_shape[1]) / 2  # width padding
    dh = (height - new_shape[0]) / 2
    top, bottom = round(dh - 0.1), round(dh + 0.1)
    left, right = round(dw - 0.1), round(dw + 0.1)
    x = torch.nn.functional.interpolate(x, size=new_shape,
                                        mode='bilinear')
    x = torch.nn.functional.pad(
        x, (left, right, top, bottom), mode='constant', value=fillvalue)
    return x


def viewImage(image):
    plt.imshow(ToPILImage()(image))
    plt.show()

if __name__ == "__main__":
    useMaskedImages = False

    # which net?
    #netname = 'market_resnet50_model_120_rank1_945'
    #netname = 'mcrv_resnet50_checkpoint_23005'
    #netname = 'mcrv_resnet50_checkpoint_202444'
    #netname = 'mcrv_resnet50_checkpoint_169785'
    netname = 'mcrv_resnet50_checkpoint_528759'

    _netsize = {
        'market_resnet50_model_120_rank1_945': 751,
        'mcrv_resnet50_checkpoint_23005': 9604,
        'mcrv_resnet50_checkpoint_202444': 9604,
        'mcrv_resnet50_checkpoint_169785': 11743,
        'mcrv_resnet50_checkpoint_528759': 11743
    }

    config_path = data_path + 'reid/reid-strong-baseline/configs/softmax_triplet_with_center.yml'
    #model_path = data_path + 'reid/market_resnet50_model_120_rank1_945.pth'
    model_path = data_path + 'reid/' + netname + '.pt'
    if netname == 'market_resnet50_model_120_rank1_945':
        model_path += 'h'

    checkin_photos = data_path + 'tracks/links/all/'
    if useMaskedImages:
        checkin_photos = data_path + 'tracks/links/masked/'  # this set of images had a mask applied

    save_path = data_path + 'reid/checkin_desciptors_' + netname
    if useMaskedImages:
        save_path += '_masked'
    save_path += '/'

    # Provision model
    dg_cfg.merge_from_file(config_path)
    # Prevent model from trying to init backbone
    dg_cfg['MODEL']['PRETRAIN_CHOICE'] = None
    dg_cfg['MODEL']['PRETRAIN_PATH'] = None

    checkpoint = torch.load(model_path, map_location="cpu")
    dg_model = build_dg_model(dg_cfg, _netsize[netname])  # 751)
    state_dict = checkpoint
    if netname != 'market_resnet50_model_120_rank1_945':
        state_dict = checkpoint['model']
    dg_model.load_state_dict(state_dict)  #.load_state_dict(checkpoint)
    _ = dg_model.eval()

    # read in the images in the folder
    images = glob.glob(checkin_photos + "*.jpg")
    # temp - limit for testing
    #images = images[0:1]
    print(f"loading {len(images)} images")

    # compute descriptors for all
    desc = {}
    for image in images:
        name = pathlib.Path(image).name
        img = Image.open(image)  # open image
        chip = ToTensor()(img).unsqueeze(0)  # create Tensor

        # normalize and letterbox
        chip = tensorletterbox(chip, height=256, width=128)
        model_in = normalize(
            chip, dg_cfg.INPUT.PIXEL_MEAN, dg_cfg.INPUT.PIXEL_STD)

        # replace the yellow?
        # tempImg = np.array(ToPILImage()(torch.squeeze(model_in)))
        # for x1 in tempImg:
        #    for x2 in x1:
        #        if x2[0] == 228 and x2[1] == 249 and x2[2] == 52:
        #            # set all these to zero
        #            x2[0] = 0
        #            x2[1] = 0
        #            x2[2] = 0
        # viewImage(tempImg)
        # model_in = ToTensor()(tempImg).unsqueeze(0)

        with torch.no_grad():
            out = dg_model(model_in)
            desc[name] = out.tolist()

    # make the output path
    pathlib.Path(save_path).mkdir(parents=True, exist_ok=True)

    # save the descriptors
    for nm in desc.keys():
        save_file = save_path + nm + '.desc'
        with open(save_file, 'wb') as of:
            pickle.dump(desc[nm], of)

    print(f"saved off descriptors: {save_path}")
