import os
import sys
from pathlib import Path

from girder import events, plugin
from girder.models.setting import Setting
from girder.utility import setting_utilities

from .client_webroot import ClientWebroot
from .constants import SETTINGS_CONST_JOBS_CONFIGS
from .event import check_existing_annotations
from .meva import Meva
from .meva_database.database import MevaDatabase
from .meva_checkin import MevaCheckIn
from .meva_detection import MevaDetection
from .meva_state.meva_state import MEVAState
from .linking_features.linking_feature import LinkedFeature


@setting_utilities.validator({SETTINGS_CONST_JOBS_CONFIGS})
def validateSettings(doc):
    """
    Handle plugin-specific system settings. Right now we don't do any
    validation.
    """
    val = doc['value']
    if val is not None:
        # TODO: replace with real schema validation
        assert 'pipelines' in val, '"piplines" missing from doc'


class GirderPlugin(plugin.GirderPlugin):
    def load(self, info):

        info["apiRoot"].meva = Meva()
        info["apiRoot"].meva_detection = MevaDetection()
        info["apiRoot"].meva_database = MevaDatabase()
        info["apiRoot"].meva_checkin = MevaCheckIn()
        info["apiRoot"].meva_state = MEVAState()
        info["apiRoot"].linked_feature = LinkedFeature()
        # Relocate Girder
        info["serverRoot"], info["serverRoot"].girder = (
            ClientWebroot(),
            info["serverRoot"],
        )
        info["serverRoot"].api = info["serverRoot"].girder.api

        events.bind(
            "filesystem_assetstore_imported",
            "check_annotations",
            check_existing_annotations,
        )

        # Create dependency on worker
        plugin.getPlugin('worker').load(info)
        Setting().set(
            'worker.api_url',
            os.environ.get('WORKER_API_URL', 'http://girder:8080/api/v1'),
        )
        Setting().set(
            'worker.broker',
            os.environ.get('WORKER_BROKER', 'amqp://guest:guest@rabbit/'),
        )
        Setting().set(
            'worker.backend',
            os.environ.get('WORKER_BACKEND', 'amqp://guest:guest@rabbit/'),
        )
