import math

from meva_server.serializers.models import Track, interpolate


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), math.ceil(len(lst) / n)):
        yield lst[i: i + math.ceil(len(lst) / n)]


def create_extraction_data(self, track_dict):
    """
    Here we need to create a file which lists of the representative
    chips for each track in the file and computed an document indicating their frame number
    and bounds so they can be extracted out and stored.
    """
    num_of_chips = 10
    output_json = {}
    for t in track_dict.values():
        track = Track(**t)
        if track.confidencePairs[0][0].startswith('person') is False and track.confidencePairs[0][0].startswith('Person') is False:
            continue
        frame_list = []
        frame_bounds = []
        for index, keyframe in enumerate(track.features):
            features = [keyframe]
            # If this is not the last keyframe, and interpolation is
            # enabled for this keyframe, interpolate
            if keyframe.interpolate and index < len(track.features) - 1:
                nextKeyframe = track.features[index + 1]
                # interpolate all features in [a,b)
                features = interpolate(keyframe, nextKeyframe)

            for feature in features:
                bounds = feature.bounds
                size = abs((bounds[3] - bounds[1])) * abs((bounds[2] - bounds[0]))
                organized_frame = {
                    'frame': feature.frame,
                    'bounds': bounds,
                    'size': size,
                }
                print('feature attributes')
                print(feature.attributes)
                if (
                    feature.attributes is not None
                    and 'poses' in feature.attributes
                    and 'points' in feature.attributes['poses']
                ):
                    organized_frame['pose'] = feature.attributes['poses']['points']
                    if 'score' in feature.attributes['poses']:
                        organized_frame['score'] = float(feature.attributes['poses']['score'])
                    else:
                        organized_frame['score'] = 1.0
                else: 
                    organized_frame['score'] = 0.0
                frame_bounds.append(organized_frame)
        # Go through the sparse array of features and split into 10 time based subdivisions
        sections = list(chunks(frame_bounds, num_of_chips))
        # now find the largest keypoint in each of those chunks
        print(f" Section Length: {len(sections)}")
        for section_chunk in sections:
            sorted_section = sorted(section_chunk, key=lambda item: item['score'])
            representative_frame = sorted_section[0]
            frame_list.append(representative_frame)
        output_json[track.trackId] = frame_list

    return output_json
