import datetime
from girder.constants import SortDir
from girder.exceptions import ValidationException, RestException
from girder.models.model_base import Model
from girder import events
from dateutil import parser


class LinkingFeature(Model):
    # This is NOT an access controlled model; it is expected that all endpoints
    # will be sensibly guarded instead.
    def __init__(self):
        # Do the bindings before calling __init__(), in case a derived class
        # wants to change things in initialize()
        super().__init__()

    def initialize(self):
        self.name = 'LinkingFeature'
        self.userId = ''
        self.folderId = ''
        self.linked = False
        self.sourceGirderId = ''
        self.sourceTrackId = ''
        self.sourceClipFilename = ''  # Clip
        self.linkedActor = ''
        self.textFeatures = []
        self.time= 0
        self.login = ''
        self.featureDrawings = (
            []
        )  # array of dicts of { identifier: f||b id: girderItemId}
        self.properties = []
        self.linkReason = ''

        self.ensureIndices(
            [
                'userId',
                'created',
                (
                    [
                        ('created', SortDir.ASCENDING),
                    ],
                    {},
                ),
            ]
        )

    def createFeature(self, user, folder, data):
        created = datetime.datetime.utcnow()
        time = 0
        if 'time' in data.keys():
            time = data['time']
        linked_feature = dict(
            created=created,
            userId=str(user['_id']),
            login=str(user['login']),
            folderId=str(folder['_id']),
            linked=data['linked'],
            sourceGirderId=data['sourceGirderId'],
            sourceTrackId=data['sourceTrackId'],
            sourceClipFilename=data['sourceClipFilename'],
            linkedActor=data['linkedActor'],
            textFeatures=data['textFeatures'],
            featureDrawings=data['featureDrawings'],
            linkReason=data['linkReason'],
            properties=data['properties'],
            time=time,
        )
        linked_feature = self.save(linked_feature)
        return linked_feature

    def validate(self, doc):
        if not doc.get('userId') or not isinstance(doc['userId'], str):
            raise ValidationException('userId must be a string')
        if 'sourceGirderId' not in doc or not isinstance(doc['sourceGirderId'], str):
            raise ValidationException('sourceGirderId must be a string')
        if not doc.get('sourceClipFilename') or not isinstance(
            doc['sourceClipFilename'], str
        ):
            raise ValidationException('sourceClipFilename must be a string')
        return doc
