import json
from typing import Any, Dict, List, Mapping, Optional, Tuple, Union

import numpy
from girder.models.file import File

from meva_server.serializers.models import Feature, Track, interpolate
from meva_server.serializers.viame import create_geoJSONFeature

coco_part_names = [
    'nose',
    'leftEye',
    'rightEye',
    'leftEar',
    'rightEar',
    'leftShoulder',
    'rightShoulder',
    'leftElbow',
    'rightElbow',
    'leftWrist',
    'rightWrist',
    'leftHip',
    'rightHip',
    'leftKnee',
    'rightKnee',
    'leftAnkle',
    'rightAnkle',
]
pairs = [
    [0, 1],
    [0, 2],
    [1, 3],
    [2, 4],
    [5, 6],
    [5, 7],
    [7, 9],
    [6, 8],
    [8, 10],
    [11, 12],
    [11, 13],
    [13, 15],
    [12, 14],
    [14, 16],
    [6, 12],
    [5, 11],
]


def convert_alphapose_tracks(file):
    alpha_json = json.loads(
        b"".join(list(File().download(file, headers=False)())).decode()
    )
    # organize file by ID and frame Numbers
    tracks = {}
    for image in alpha_json.keys():
        for detection in alpha_json[image]:
            frame = round(int(image.replace('.jpg', '')))
            pose = numpy.array(detection['keypoints']).reshape(-1, 3)[:, :3]
            score = detection['scores']
            trackId = detection['idx']
            if trackId not in tracks.keys():
                track = Track(begin=frame, end=frame, trackId=trackId)
                track.confidencePairs = [[f'person_{trackId}', score / 5.0]]
                tracks[trackId] = track
            track = tracks[trackId]
            track.begin = min(frame, track.begin)
            track.end = max(track.end, frame)
            attributes = {"poses": {"points": {}, "score": score}}
            counter = 0
            x_min = float('inf')
            x_max = float('-inf')
            y_min = float('inf')
            y_max = float('-inf')
            print(pose)
            for key in coco_part_names:
                x = round(pose[counter][0])
                y = round(pose[counter][1])
                attributes['poses']['points'][key] = [x, y]
                x_max = max(x_max, x)
                y_max = max(y_max, y)
                x_min = min(x_min, x)
                y_min = min(y_min, y)
                counter = counter + 1
            bounds = [x_min, y_min, x_max, y_max]
            feature: Feature = Feature(
                frame=frame, bounds=bounds, attributes=attributes or None
            )
            track.features.append(feature)

    return {trackId: track.dict(exclude_none=True) for trackId, track in tracks.items()}
