import json
import os
import click
import shutil
from pathlib import Path
import random

# Read in JSON file to get list of video names
def load_actors(filename):
    f = open(filename, 'r')
    json_string = f.read()
    actor_file = json.loads(json_string)
    results = actor_file['pageResults']
    actor_map = {}
    for item in results:
        actorId = item['actor']
        date = item['date']
        outfitId = item['outfit']
        if actorId not in actor_map.keys():
            actor_map[actorId] = {}
        if date not in actor_map[actorId].keys():
            actor_map[actorId][date] = {}
        if outfitId not in actor_map[actorId][date].keys():
            actor_map[actorId][date][outfitId] = True
    return actor_map

def load_links(filename):
    f = open(filename, 'r')
    json_string = f.read()
    json_links = json.loads(json_string)
    return json_links

def get_random_outfit(actor_map, actorId, date, outfitId):
    actor_obj = actor_map[actorId]
    new_date = date
    new_outfitId = outfitId
    while new_date == date and new_outfitId == outfitId:
        new_date = random.choice(list(actor_obj.keys()))
        new_outfitId = random.choice(list(actor_obj[new_date].keys()))
        # print('Trying to find non matching date_outfitId')
        # print(f'{date} == {new_date} and {outfitId} == {new_outfitId}')

    return new_date, new_outfitId

@click.command(name="Update", help="Takes a trackLink JSON and updates all track files")
@click.argument('actorfile')
@click.argument('linkfile')
@click.argument('limit')
@click.argument('totalactors')
def load_data(actorfile, linkfile, limit, totalactors):
    actor_map = load_actors(actorfile)
    links = load_links(linkfile)['pageResults']
    # now we randomly select 20 linked actors and a filename for each of them
    limit = int(limit)
    totalActors = int(totalactors)
    linking_list = []
    for current in range(limit):
        print(f'Adding Current Random {current}')
        rand_actor = random.choice(links)
        print(f'Actor has {len(actor_map[rand_actor["actorId"]].keys())} dates')
        rand_file = random.choice(rand_actor['links'])
        sourceClipFilename = rand_file['name']
        sourceTrackId = rand_file['trackId']
        actor_list = []
        actor_list.append({
            "actorId": rand_actor['actorId'],
            "date": rand_actor['date'],
            "outfitId": rand_actor["outfitId"]
        })
        # now we need to add another image of this actor from the list
        actor = actor_map[rand_actor['actorId']]
        add_source_date, add_source_outfitId = get_random_outfit(actor_map, rand_actor['actorId'], rand_actor['date'], rand_actor['outfitId'])
        actor_list.append({
            "actorId": rand_actor['actorId'],
            "date": add_source_date,
            "outfitId": add_source_outfitId
        })
        # Next we loop through and add X additional actors and an alternative outfit to the actor list
        check_existing_actors = { rand_actor['actorId']: True}
        actor_limit = 9
        new_actor = rand_actor['actorId']
        for new_actor_count in range(int(totalActors / 2)):
            print(f'Adding in Actor {new_actor_count}')
            redo = False
            while new_actor in check_existing_actors.keys() or redo:
                new_actor = random.choice(list(actor_map.keys()))
                if len(actor_map[new_actor].keys()) == 1 and len(actor_map[new_actor][list(actor_map[new_actor].keys())[0]].keys()) == 1:
                    redo = True
                else:
                    redo = False
            new_date = random.choice(list(actor_map[new_actor].keys()))
            new_outfit = random.choice(list(actor_map[new_actor][new_date].keys()))
            actor_list.append({
                "actorId": new_actor,
                "date": new_date,
                "outfitId": new_outfit
            })
            # now we add the alternative outfit
            new_add_date, new_add_outfitId = get_random_outfit(actor_map, new_actor, new_date, new_outfit)
            actor_list.append({
                "actorId": new_actor,
                "date": new_add_date,
                "outfitId": new_add_outfitId
            })
            check_existing_actors[new_actor] = True
        # create our final item
        random.shuffle(actor_list)
        linking_list.append({
            "sourceClipFilename": sourceClipFilename,
            "sourceTrackId": sourceTrackId,
            "Actors": actor_list
        })
    
    with open('linkingList.json', 'w') as outfile:
        json.dump(linking_list, outfile)



if __name__ == '__main__':
    load_data()

