from girder_client import GirderClient
from girder_worker.app import app
from girder_worker.task import Task
from girder_worker.utils import JobManager, JobStatus
from urllib import request
from pathlib import Path
import os
import gpxpy
import math
import numpy as np
import random
import pytz
import uuid
import shutil
import time
import pandas as pd
from itertools import combinations
from datetime import date, datetime, timedelta
from zipfile import ZipFile
from sqlalchemy import and_, case, create_engine, not_, or_
from sqlalchemy.orm import (
    contains_eager,
    joinedload,
    lazyload,
    selectinload,
    sessionmaker,
    subqueryload,
)
import logging
import subprocess
import sys
from sqlalchemy.sql.elements import Null
from sqlalchemy.sql.expression import func, select
from meva_utils.models import (
    ActorDisplayChip,
    Base,
    Camera,
    Detection,
    Response,
    Source,
    Track,
    Gps,
    MustNotLink,
    Afinity
)
from subprocess import Popen
from meva_tasks.utils import (
    stream_subprocess,
)
import tempfile


# TODO: these should probably not be hard-coded
# some default values that can be overriden by the args passed in
_latitude = 39.04977294
_longitude = -85.52924953
_height = 205

# time sync error captures uncertainty in the times assigned to
# videos with respect to other videos or gps logs and we assume that
# the true time is equal the reported time plus or minus the time sync error
_timeSyncError = 3  # seconds

# threshold value for score from getScoreForPositions encoding the threshold
# for certainty of overlap of the localization ellipsoids
_scoreThreshold = 1e-4  # unitless

# default start value for comparing to a minimum val
_infinity = 9999999999.0

# default value used to ref covariance in gps logs
_meters_err = 9

# variables used for saving memory in the pool for mp
_track_detections = {}
_track_gpslogs = {}
_detectMinTime = {}
_detectMaxTime = {}

@app.task(bind=True, ignore_result=True)
def local_must_not_link_task(self: Task):
    manager: JobManager = self.job_manager
    gc: GirderClient = self.girder_client
    command = [
        "cd /home/data_scripts && python3",
        "populateMustNotLink.py",
        "/data/db/meva.db",
    ]
    process_err_file = tempfile.TemporaryFile()
    cmd = " ".join(command)
    manager.write(f"Running command: {cmd}\n", forceFlush=True)
    process = Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=process_err_file,
        shell=True,
        executable='/bin/bash',
    )
    process_err_file = tempfile.TemporaryFile()

    stream_subprocess(process, self, manager, process_err_file)
    if self.canceled:
        return
    # Now we upload the data and placed Item Ids in there to make it easier to load info for display
    manager.updateStatus(JobStatus.PUSHING_OUTPUT)
