import argparse
import json
import os
from typing import Dict, List
from zipfile import ZipFile
from PIL import Image


def start_actor_extraction(zip_file, actorJSON=None):
    actorjson = {}
    existing_files = []
    print(actorJSON)
    if actorJSON is not None and actorJSON != '' :
        file_obj = open(actorJSON)
        actorjson = json.loads(file_obj.read())
    print(actorjson)
    os.mkdir('actors')
    bad_filenames = []
    with ZipFile(zip_file, 'r') as zipObj:
        listOfFileNames = zipObj.namelist()
        for fileName in listOfFileNames:
            baseName = fileName.replace('.jpg', '')
            print(f'FILENAME: {fileName}')
            data = baseName.split('-')
            if len(data) != 5:
                bad_filenames.append(fileName)
                continue
            actor = data[0]
            gps = data[1]
            date = data[2]
            outfit = data[3]
            key_name = f'{actor}-{gps}-{date}-{outfit}'
            # print(f'key_name: {key_name}')
            if key_name in actorjson.keys():
                print(f'skipping actor: {key_name}')
                continue
            front_back = str(data[4]).lower()
            if not os.path.exists(f'./actors/{actor}'):
                os.mkdir(f'./actors/{actor}')
            if not os.path.exists(f'./actors/{actor}/{date}'):
                os.mkdir(f'./actors/{actor}/{date}')
            if not os.path.exists(f'./actors/{actor}/{date}/{outfit}'):
                os.mkdir(f'./actors/{actor}/{date}/{outfit}')
            zipObj.extract(fileName, f'./actors/{actor}/{date}/{outfit}')
            full_size = fileName.replace('.jpg', '-full.jpg')
            os.rename(
                f'./actors/{actor}/{date}/{outfit}/{fileName}',
                f'./actors/{actor}/{date}/{outfit}/{full_size}',
            )
            image = Image.open(f'./actors/{actor}/{date}/{outfit}/{full_size}')
            size = 300, 600
            image.thumbnail(size, Image.ANTIALIAS)
            thumbnail = fileName.replace('.jpg', '-thumb.jpg')
            image.save(
                f'./actors/{actor}/{date}/{outfit}/{thumbnail}',
                optimize=True,
                quality=95,
            )
        print('BAD FILENAMES:')
        for item in bad_filenames:
            print(item)


def main(args):
    start_actor_extraction(args.zip_file, args.actorJSON)


def parse_args():
    parser = argparse.ArgumentParser(description='Actor Extraction')

    parser.add_argument('--zip-file', default='', help='Zip file')
    parser.add_argument('--actorJSON', default='', help='Zip file')

    return parser.parse_args()


if __name__ == '__main__':
    args = parse_args()
    main(args)
