
import boto3
import os
import tempfile
from botocore import UNSIGNED
from botocore.client import Config
from pathlib import Path
from subprocess import Popen
from sys import meta_path

from girder_client import GirderClient
from girder_worker.app import app
from girder_worker.task import Task
from girder_worker.utils import JobManager
from urllib import request

def convert_to_bucket_key(filename):
    s3_filename = filename.replace('s3://', '')
    s3_split = s3_filename.split('/')
    bucket = s3_split[0]
    key = s3_filename.replace(bucket + '/', '')
    return {"bucket": bucket, "key": key}


@app.task(bind=True, acks_late=True)
def import_meva_from_url(self: Task, folder, filename, videoURL, tracksURL, descriptorURL):
    folderId = str(folder["_id"])
    manager: JobManager = self.job_manager
    gc: GirderClient = self.girder_client

    tracks_location = None
    descriptor_location = None
    if videoURL.startswith('http') is True:
        video_location = request.urlretrieve(videoURL)[0]
    elif videoURL.startswith('s3') is True:
        temp_dir = tempfile.mkdtemp()
        video_location = os.path.join(temp_dir, filename)
        s3 = boto3.client('s3', config=Config(signature_version=UNSIGNED))
        data = convert_to_bucket_key(videoURL)
        print(f"bucket: {data['bucket']} key: {data['key']}")
        with open(video_location, "wb") as f:
            s3.download_fileobj(data['bucket'], data['key'], f)

    if tracksURL is not None:
        if tracksURL.startswith('http') is True:
            tracks_location = request.urlretrieve(tracksURL)[0]
            print('Tracks Location:')
            print(tracks_location)
        elif tracksURL.startswith('s3') is True:
            temp_dir = tempfile.mkdtemp()
            tracks_location = filename.replace('.avi', '.json')
            tracks_location = os.path.join(temp_dir, tracks_location)
            s3 = boto3.client('s3', config=Config(signature_version=UNSIGNED))
            data = convert_to_bucket_key(tracksURL)
            with open(tracks_location, "wb") as f:
                s3.download_fileobj(data['bucket'], data['key'], f)

    if descriptorURL is not None:
        if descriptor_location.startswith('http') is True:
            descriptor_location = request.urlretrieve(descriptorURL)[0]
        elif descriptor_location.startswith('s3') is True:
            temp_dir = tempfile.mkdtemp()
            descriptor_location = filename.replace('.avi', '.descriptor')
            descriptor_location = os.path.join(temp_dir, descriptor_location)
            s3 = boto3.client('s3', config=Config(signature_version=UNSIGNED))
            data = convert_to_bucket_key(descriptorURL)
            with open(descriptor_location, "wb") as f:
                s3.download_fileobj(data['bucket'], data['key'], f)

    video_folder = gc.createFolder(folderId, filename)
    gc.addMetadataToFolder(str(video_folder["_id"]), {"type": "video"})
    if tracks_location is not None:
        tracks_item = gc.uploadFileToFolder(str(video_folder["_id"]), tracks_location, filename=filename.replace('.avi', '.json'))
    if descriptorURL is not None:
        descriptor_item = gc.uploadFileToFolder(str(video_folder["_id"]), descriptor_location)
        gc.addMetadataToItem(descriptor_item["itemId"], {"descriptors": str(str(video_folder["_id"]))})
    if video_location[0] is not None:
        video_item = gc.uploadFileToFolder(str(video_folder["_id"]), video_location, filename=filename)
        # should be as simple as doing a post process now
        gc.sendRestRequest('POST', f'meva/postprocess/{str(video_folder["_id"])}')
