import argparse
import json
from meva_server.meva_database.database import MevaDatabase
from meva_server.meva_database.init_database import run_database_init
from meva_server.meva_database.utils import getCameraModelMap
import logging
import os
from pathlib import Path
import uuid
from xml.dom.minidom import parse
import zipfile

parser = argparse.ArgumentParser(
    description='Script to import tracks from json'
    ' and check-in photos from zip files into MEVA database.'
)
parser.add_argument('databasePath', help='path to database file')
parser.add_argument(
    'dataDir', help='path to directory that holds the' 'data to be imported'
)
parser.add_argument(
    "metadataFolder",
    help="path to meta data: https://gitlab.kitware.com/meva/meva-data-repo/-/blob/master/metadata/"
)
parser.add_argument(
    "npyPath", help="folder containing the npy files (in folders by source name)"
)

args = parser.parse_args()

database_path = Path(args.databasePath)
data_dir = Path(args.dataDir)

os.environ['MEVA_DATABASE_URI'] = 'sqlite:///' + str(database_path)

run_database_init()

meva_db = MevaDatabase()

# turn off sqlalchemy logs
for name in logging.root.manager.loggerDict:
    if 'sqlalchemy' in name:
        logger = logging.getLogger(name)
        logger.disabled = True

camera_model_lookup_fname = (
    args.metadataFolder + "meva-clip-camera-and-time-table.txt"
)
camera_model_map = getCameraModelMap(camera_model_lookup_fname)

track_files = data_dir.glob('*.json')

for tf in track_files:
    print('Loading track file ', tf.name)
    fields = tf.name.split('.')

    folder_obj = {
        'name': tf.name,
        'meta': {
            'MEVA_Meta': {
                "date": fields[0],
                "startTime": fields[1],
                "endTime": fields[2],
                "location": fields[3],
                "cameraId": fields[4],
            }
        },
        'fps': 30,
    }

    with open(tf) as f:
        track_data = json.load(f)

    meva_db._add_tracks(track_data, uuid.uuid4().hex, folder_obj,
                        camera_model_map, args.npyPath)

zip_files = list(data_dir.glob('*.zip'))

zip_file = zipfile.ZipFile(zip_files[0])

actor_data = {}
for zf in zip_file.namelist():
    basename = Path(zf).with_suffix('')
    print('Loading ',basename, ' from zip file ', zip_file.filename)
    if str(basename).endswith('-b'):
        fields = str(basename).split('-')
        actor_data[basename] = {
            'filename': str(basename) + '-full.jpg',
            'actor': fields[0],
            'date': fields[1],
            'gps': fields[2],
            'outfit': fields[3],
        }

meva_db._add_actor_chips(actor_data)

gps_files = list(data_dir.glob('*.gpx'))

for gf in gps_files:
    print('Loading gpx file ', gf.name)
    with open(gf) as gpx_file:
        meva_db._add_gps(gpx_file)

krtd_files = list(data_dir.glob('*.krtd'))

for cam_f in krtd_files:
    print('Loading krtd file ', cam_f.name)
    with open(cam_f) as f:
        meva_db._add_krtd_camera(cam_f.name, f.read())
